///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                       //
//    e3dMATH 3D Math Library                                                                                            //
//    version: 0.1a                                                                                                      //
//    author : Achille Peternier                                                                                         //
//             eXence software (C) 2004                                                                                  //
//    created: 15/06/2004						                                                                               //
//                                                                                                                       //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef __E3DMATH_3D_MATH__
#define __E3DMATH_3D_MATH__

//////////////////
// FIX: & speedup
//////////////////



///////////////
// #INCLUDES //
///////////////
#include <stdlib.h>
#include <math.h>
#include <float.h>



/////////////
// #DEFINE //
/////////////

	// Debugging, consistency:
	#define E3M_SAFEMODE		// Additional hardcoded checks, improved accuracy, remove to speedup code
	#define E3M_SSE         // Optional SSE alignement (not used so far)
	#if defined(WIN32) || defined(UNDER_CE)
	   #define E3M_API __declspec(dllexport)
	#else // LINUX:
	   #define E3M_API
	   #define _isnan isnan
	#endif
	#define E3M_PI          3.14159265358979323846f
	#define E3M_TODEC(x)    (x/E3M_PI) * 180.0f
	#define E3M_TORAD(x)    (x/180.0f) * E3M_PI
	
   // Fixed math:
   #define E3M_PRECISION   16
   #define E3M_INT32       int
   #define E3M_INT64       long long
   #define E3M_ONE         0x1000



/////////////
// CLASSES //
/////////////

///////////////////////////////////////////////
typedef class E3M_FIXED // Fixed-point class //
///////////////////////////////////////////////
{      
/////////////
   public: //
/////////////

   // Components:
   E3M_INT32 d;
   
   // Constructors:
   explicit E3M_FIXED(E3M_INT32 i, bool)         { d = i; }
   E3M_FIXED()                                   { d = 0; }
   E3M_FIXED(const E3M_FIXED& f)                 { d = f.d; }
   E3M_FIXED(const float f)                      { d = static_cast<E3M_INT32>(f * static_cast<float>(E3M_ONE)); }
   E3M_FIXED(E3M_INT32 i)                        { d = i << E3M_PRECISION; }
   
   // Operators:
   operator E3M_INT32() const                    { return d >> E3M_PRECISION; }
   operator float() const                        { return d * (1.0f/static_cast<float>(E3M_ONE)); }   
   bool operator == (const E3M_FIXED& o) const   { return d == o.d; }
   bool operator != (const E3M_FIXED& o) const   { return d != o.d; }
   bool operator <  (const E3M_FIXED& o) const   { return d <  o.d; }
   bool operator >  (const E3M_FIXED& o) const   { return d >  o.d; }
   bool operator <= (const E3M_FIXED& o) const   { return d <= o.d; }
   bool operator >= (const E3M_FIXED& o) const   { return d >= o.d; }
   
   // Math operators:
   E3M_FIXED operator - () const                   { return E3M_FIXED(-d, true); }
   E3M_FIXED operator + (const E3M_FIXED& o) const { return E3M_FIXED(d + o.d, true); }
   E3M_FIXED operator - (const E3M_FIXED& o) const { return E3M_FIXED(d - o.d, true); }
   E3M_FIXED operator * (const E3M_FIXED& o) const { return E3M_FIXED(static_cast<E3M_INT32>(((E3M_INT64) d * (E3M_INT64) o.d) >> E3M_PRECISION), true); }
 //  E3M_FIXED operator / (const E3M_FIXED& o) const { return (*this * o.Inverse(o)); }
    
   //Assignements:
   E3M_FIXED& operator = (const E3M_FIXED& o)    { d = o.d; return *this; }   
   E3M_FIXED& operator += (const E3M_FIXED& o)   { d += o.d; return *this; }
   E3M_FIXED& operator -= (const E3M_FIXED& o)   { d -= o.d; return *this; }
   E3M_FIXED& operator *= (const E3M_FIXED& o)   { d = static_cast<E3M_INT32>(((E3M_INT64) d * (E3M_INT64) o.d) >> E3M_PRECISION); return *this; }  
   //E3M_FIXED& operator /= (const E3M_FIXED& o)   { return *this *= o.inverse(); }
   E3M_FIXED& operator >>= (const E3M_INT32 s)   { d >>= s; return *this; }
   E3M_FIXED& operator <<= (const E3M_INT32 s)   { d <<= s; return *this; }
   static float fromValue(E3M_INT32 i)           { return E3M_FIXED(i, true); }           
   
   // Other:   
  // E3M_FIXED inverse()                           { E3M_FIXED inv = *this; inv = Inverse(inv); return inv; }
  /* E3M_FIXED Inverse(E3M_FIXED a)
   {
      E3M_INT32 exp;
      E3M_FIXED x; 
      static const E3M_INT32 __gl_rcp_tab[] = {0x8000, 0x71c7, 0x6666, 0x5d17, 0x5555, 0x4ec4, 0x4924, 0x4444};
      if (a == E3M_FIXED(0)) 
            return 0x7fffffff;
      bool sign = false;
      
      if (a < E3M_FIXED(0))
      {
            sign = true;            
            a = -a;
      }      
      
      x = a;
      exp = 31;
      if (x & 0xffff0000)  { exp -= 16; x >>= 16; }
      if (x & 0xff00)      { exp -= 8;  x >>= 8; }
      if (x & 0xf0)        { exp -= 4;  x >>= 4; }
      if (x & 0xc)         { exp -= 2;  x >>= 2; }
      if (x & 0x2)         { exp -= 1; }
      x = __gl_rcp_tab[(a>>(28-exp))&0x7]<<2;
      exp -= 16;
      if (exp <= 0)
		   x >>= -exp;
      else
		   x <<= exp;
		   
		x = x * (E3M_FIXED(1)*E3M_FIXED(2) - (a * x));
		x = x * (E3M_FIXED(1)*E3M_FIXED(2) - (a * x));
	   
	   if (sign)
	         return -x;
	   else
	         return x;
   }*/   
} E3M_FIXED;





///////////////////////////////////////////////////////////////
typedef class E3M_API E3M_SSE E3MVECTOR2D // 2D vector class //
///////////////////////////////////////////////////////////////
{
/////////////
   public: //
/////////////

	// Components:
	union
	{
		float xy[2];
		float uv[2];
		struct { float x, y; };
		struct { float u, v; };
	};

   // Constructors:
	inline E3MVECTOR2D()							  { x=y=0.0f; }
	inline E3MVECTOR2D(float X, float Y)     { x=X; y=Y; }
	inline E3MVECTOR2D(double X, double Y)   { x=(float)X; y=(float)Y; }
	inline E3MVECTOR2D(int X, int Y)			  { x=(float)X; y=(float)Y; }
	inline E3MVECTOR2D(const E3MVECTOR2D& v) { x=v.x; y=v.y; } 
	inline E3MVECTOR2D(float af[2])			  { x=af[0]; y=af[1]; }
	
	// Operators:
	inline float& operator[](int i) 
	{
#ifdef E3M_SAFEMODE
			if (i<0 || i>1) exit(1);
#endif 
			return (&x)[i]; 
	}     
	inline const float& operator[](int i) const 
	{ 
#ifdef E3M_SAFEMODE
			if (i<0 || i>1) exit(1);
#endif
			return (&x)[i]; 
	} 	
	inline operator float*() { return(&x); }   	
	inline E3MVECTOR2D operator-() const { return E3MVECTOR2D(-x,-y); } 
	inline E3MVECTOR2D operator+() const { return E3MVECTOR2D(x, y); }    		
	inline E3MVECTOR2D operator+(const float k)      const { return E3MVECTOR2D(x+k, y+k); }
	inline E3MVECTOR2D operator+(const E3MVECTOR2D& v) const { return E3MVECTOR2D(x+v.x, y+v.y); }
	inline E3MVECTOR2D operator-(const float k)      const { return E3MVECTOR2D(x-k, y-k); }
	inline E3MVECTOR2D operator-(const E3MVECTOR2D& v) const { return E3MVECTOR2D(x-v.x, y-v.y); }		
	inline E3MVECTOR2D operator*(const float k)      const { return E3MVECTOR2D(x*k, y*k); }
	inline E3MVECTOR2D operator*(const E3MVECTOR2D& v) const { return E3MVECTOR2D(x*v.x, y*v.y); }	
	inline E3MVECTOR2D operator/(const float k)      const { return E3MVECTOR2D(x/k, y/k); }
	inline E3MVECTOR2D operator/(const E3MVECTOR2D& v) const { return E3MVECTOR2D(x/v.x, y/v.y); } 	 
	inline float operator%(const E3MVECTOR2D& v) const { return (x*v.x + y*v.y); } // Dot product	
	inline E3MVECTOR2D operator-=(const E3MVECTOR2D& v) { x-=v.x; y-=v.y; return *this; }
	inline E3MVECTOR2D operator+=(const E3MVECTOR2D& v) { x+=v.x; y+=v.y; return *this; }
	inline E3MVECTOR2D operator*=(const float k)      { x*=k; y*=k; return *this; }
	inline E3MVECTOR2D operator*=(const E3MVECTOR2D& v) { x*=v.x; y*=v.y; return *this; } 
	inline E3MVECTOR2D operator/=(const float k)      { x/=k; y/=k; return *this; }
	inline E3MVECTOR2D operator/=(const E3MVECTOR2D& v) { x/=v.x; y/=v.y; return *this; }	
	inline int operator==(const E3MVECTOR2D& v) const { return ((v.x==x)&&(v.y==y)); }
	inline int operator!=(const E3MVECTOR2D& v) const { return ((v.x!=x)||(v.y!=y)); }
	
	// Assignement:   
	inline void reset() { x=y=0.0f; }	
   inline E3MVECTOR2D set(float X, float Y)   { x=X; y=Y; return *this; }
   inline E3MVECTOR2D set(double X, double Y) { x=(float)X; y=(float)Y; return *this; }
   inline E3MVECTOR2D set(int X, int Y)       { x=(float)X; y=(float)Y; return *this; }
   inline E3MVECTOR2D set(float af[2])        { x=af[0]; y=af[1]; return *this; }  
	
	// Base functions:
	inline E3MVECTOR2D lerp(E3MVECTOR2D &v, const float k) {	return (*this + ((v - *this) * k)); }
	inline float length() const 
	{	
#ifdef E3M_SAFEMODE
		return (float) sqrt(x*x+y*y); 
#else
		return sqrtf(x*x+y*y);
#endif
	}
	inline float squaredLength() const { return x*x+y*y; }	
	inline E3MVECTOR2D setLength(const float k) { register float _ratio=k/length(); x*=_ratio;	y*=_ratio; return *this; }
	inline E3MVECTOR2D normalize() { register float len=length(); if (len==0.0f) len=1.0f; x/=len; y/=len; return *this; }	
	
	// Advanced functions:
	inline double angle(const E3MVECTOR2D& v)		
	{
#ifdef E3M_SAFEMODE	
		double dotProduct = *this % v;					
		double vectorsMagnitude = length() * v.length();
		double angle = acos(dotProduct / vectorsMagnitude);		
		if (_isnan(angle))
				return 0;			
		return(angle);
#else
		float dotProduct = *this % v;					
		float vectorsMagnitude = length() * v.length();
		float angle = acosf(dotProduct / vectorsMagnitude);		
		if (_isnan(angle))
				return 0;			
		return((double) angle);
#endif
	}
	inline float distance(const E3MVECTOR2D& v)
	{	
		register float dx = v.x - x;
		register float dy = v.y - y;	
#ifdef E3M_SAFEMODE	
		return (float) sqrt(dx*dx + dy*dy);
#else
		return sqrtf(dx*dx + dy*dy);
#endif
	}
} E3M_SSE E3MVECTOR2D; // 2D vector class





///////////////////////////////////////////////////////////////
typedef class E3M_API E3M_SSE E3MVECTOR3D // 3D vector class //
///////////////////////////////////////////////////////////////
{
/////////////
   public: //
/////////////

   // Components:
   union
	{
		float xyz[3];
		float rgb[3];
		float uvw[3];
		struct { float x, y, z; };
		struct { float r, g, b; };
		struct { float u, v, w; };		
	};   

   // Constructors:
	inline E3MVECTOR3D()				      				 { x=y=z=0.0f; }
	inline E3MVECTOR3D(float X, float Y, float Z)    { x=X; y=Y; z=Z; }
	inline E3MVECTOR3D(double X, double Y, double Z) { x=(float)X; y=(float)Y; z=(float)Z; }
	inline E3MVECTOR3D(int X, int Y, int Z)			 { x=(float)X; y=(float)Y; z=(float)Z; }
	inline E3MVECTOR3D(const E3MVECTOR2D& v)			 { x=v.x; y=v.y; z=0.0f; } 
	inline E3MVECTOR3D(const E3MVECTOR3D& v)			 { x=v.x; y=v.y; z=v.z; } 
	inline E3MVECTOR3D(float af[3])						 { x=af[0]; y=af[1]; z=af[2]; }
	
	// Operators:
	inline float& operator[](int i) 
	{
#ifdef E3M_SAFEMODE
			if (i<0 || i>2) exit(1);
#endif 
			return (&x)[i]; 
	}     
	inline const float& operator[](int i) const 
	{ 
#ifdef E3M_SAFEMODE
			if (i<0 || i>2) exit(1);
#endif
			return (&x)[i]; 
	} 	
	inline operator float*() { return(&x); }   	
	inline E3MVECTOR3D operator-() const { return E3MVECTOR3D(-x,-y,-z); } 
	inline E3MVECTOR3D operator+() const { return E3MVECTOR3D(x, y, z); }    		
	inline E3MVECTOR3D operator+(const float k)      const { return E3MVECTOR3D(x+k, y+k, z+k); }
	inline E3MVECTOR3D operator+(const E3MVECTOR3D& v) const { return E3MVECTOR3D(x+v.x, y+v.y, z+v.z); }
	inline E3MVECTOR3D operator-(const float k)      const { return E3MVECTOR3D(x-k, y-k, z-k); }
	inline E3MVECTOR3D operator-(const E3MVECTOR3D& v) const { return E3MVECTOR3D(x-v.x, y-v.y, z-v.z); }		
	inline E3MVECTOR3D operator*(const float k)      const { return E3MVECTOR3D(x*k, y*k, z*k); }
	inline E3MVECTOR3D operator*(const E3MVECTOR3D& v) const { return E3MVECTOR3D(x*v.x, y*v.y, z*v.z); }	
	inline E3MVECTOR3D operator/(const float k)      const { return E3MVECTOR3D(x/k, y/k, z/k); }
	inline E3MVECTOR3D operator/(const E3MVECTOR3D& v) const { return E3MVECTOR3D(x/v.x, y/v.y, z/v.z); } 			
	inline float operator%(const E3MVECTOR3D& v) const { return (x*v.x + y*v.y + z*v.z); } // Dot product
	inline E3MVECTOR3D operator^(const E3MVECTOR3D& v) const { return E3MVECTOR3D(y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x); } // Cross product	
	inline E3MVECTOR3D operator=(const E3MVECTOR2D& v) { x=v.x; y=v.y; z=0.0f; return *this; }
	inline E3MVECTOR3D operator-=(const E3MVECTOR3D& v) { x-=v.x; y-=v.y; z-=v.z; return *this; }
	inline E3MVECTOR3D operator+=(const E3MVECTOR3D& v) { x+=v.x; y+=v.y; z+=v.z; return *this; }
	inline E3MVECTOR3D operator*=(const float k)      { x*=k; y*=k; z*=k; return *this; }
	inline E3MVECTOR3D operator*=(const E3MVECTOR3D& v) { x*=v.x; y*=v.y; z*=v.z; return *this; } 
	inline E3MVECTOR3D operator/=(const float k)      { x/=k; y/=k; z/=k; return *this; }
	inline E3MVECTOR3D operator/=(const E3MVECTOR3D& v) { x/=v.x; y/=v.y; z/=v.z; return *this; }	
	inline int operator==(const E3MVECTOR3D& v) const { return ((v.x==x)&&(v.y==y)&&(v.z==z)); }
	inline int operator!=(const E3MVECTOR3D& v) const { return ((v.x!=x)||(v.y!=y)||(v.z!=z)); }
	
	// Assignement:   
	inline void reset() { x=y=z=0.0f; }	
   inline E3MVECTOR3D set(float X, float Y, float Z)    { x=X; y=Y; z=Z; return *this; }
   inline E3MVECTOR3D set(double X, double Y, double Z) { x=(float)X; y=(float)Y; z=(float)Z; return *this; }
   inline E3MVECTOR3D set(int X, int Y, int Z)          { x=(float)X; y=(float)Y; z=(float)Z; return *this; }
   inline E3MVECTOR3D set(float af[3])                  { x=af[0]; y=af[1]; z=af[2]; return *this; }  
	
	// Base functions:
	inline E3MVECTOR3D lerp(E3MVECTOR3D &v, const float k) {	return (*this + ((v - *this) * k)); }
	inline float length() const 
	{	
#ifdef E3M_SAFEMODE
		return (float) sqrt(x*x+y*y+z*z); 
#else
		return sqrtf(x*x+y*y+z*z);
#endif
	}
	inline float squaredLength() const { return x*x+y*y+z*z; }	
	inline E3MVECTOR3D setLength(const float k) { register float _ratio=k/length(); x*=_ratio;	y*=_ratio; z*=_ratio; return *this; }
	inline E3MVECTOR3D normalize() { register float len=length(); if (len==0.0f) len=1.0f; x/=len; y/=len; z/=len; return *this; }	
	
	// Advanced functions:
	inline double angle(const E3MVECTOR3D& v)	
	{
#ifdef E3M_SAFEMODE
		double dotProduct = *this % v;					
		double vectorsMagnitude = length() * v.length();
		double angle = acos(dotProduct / vectorsMagnitude);		
		if (_isnan(angle))
				return 0;			
		return(angle);
#else
		float dotProduct = *this % v;					
		float vectorsMagnitude = length() * v.length();
		float angle = acosf(dotProduct / vectorsMagnitude);		
		if (_isnan(angle))
				return 0;			
		return((double)angle);		
#endif
	}
	inline float distance(const E3MVECTOR3D& v)
	{	
		register float dx = v.x - x;
		register float dy = v.y - y;
		register float dz = v.z - z;
#ifdef E3M_SAFEMODE
		return (float) sqrt(dx*dx + dy*dy + dz*dz);
#else
		return sqrtf(dx*dx + dy*dy + dz*dz);
#endif
	}	
	// Matrix * Vector ...
	// Vector Matrix relative
} E3M_SSE E3MVECTOR3D; // 3D vector class





///////////////////////////////////////////////////////////////
typedef class E3M_API E3M_SSE E3MVECTOR4D // 4D vector class //
///////////////////////////////////////////////////////////////
{
/////////////
   public: //
/////////////

   // Components:
   union
	{
		float xyzw[4];
		float rgba[4];
		struct { float x, y, z, w; };
		struct { float r, g, b, a; };
		
	};   

   // Constructors:
	inline E3MVECTOR4D()				      							  { x=y=z=0.0f; w=1.0f; }
	inline E3MVECTOR4D(float X, float Y, float Z, float W)     { x=X; y=Y; z=Z; w=W; }
	inline E3MVECTOR4D(double X, double Y, double Z, double W) { x=(float)X; y=(float)Y; z=(float)Z; w=(float)W; }
	inline E3MVECTOR4D(int X, int Y, int Z, int W)	  		     { x=(float)X; y=(float)Y; z=(float)Z; w=(float)W; }
	inline E3MVECTOR4D(const E3MVECTOR2D& v)					     { x=v.x; y=v.y; z=0.0f; w=1.0f; } 
	inline E3MVECTOR4D(const E3MVECTOR3D& v)					     { x=v.x; y=v.y; z=v.z; w=1.0f; } 
	inline E3MVECTOR4D(const E3MVECTOR4D& v)					     { x=v.x; y=v.y; z=v.z; w=v.w; } 
	inline E3MVECTOR4D(float af[4])									  { x=af[0]; y=af[1]; z=af[2]; w=af[3]; }
	
	// Operators:
	inline float& operator[](int i) 
	{
#ifdef E3M_SAFEMODE
			if (i<0 || i>3) exit(1);
#endif 
			return (&x)[i]; 
	}     
	inline const float& operator[](int i) const 
	{ 
#ifdef E3M_SAFEMODE
			if (i<0 || i>3) exit(1);
#endif
			return (&x)[i]; 
	} 	
	inline operator float*() { return(&x); }   	
	inline E3MVECTOR4D operator-() const { return E3MVECTOR4D(-x,-y,-z, -w); } 
	inline E3MVECTOR4D operator+() const { return E3MVECTOR4D(x, y, z, w); }    		
	inline E3MVECTOR4D operator+(const float k)      const { return E3MVECTOR4D(x+k, y+k, z+k, w+k); }
	inline E3MVECTOR4D operator+(const E3MVECTOR4D& v) const { return E3MVECTOR4D(x+v.x, y+v.y, z+v.z, w+v.w); }
	inline E3MVECTOR4D operator-(const float k)      const { return E3MVECTOR4D(x-k, y-k, z-k, w-k); }
	inline E3MVECTOR4D operator-(const E3MVECTOR4D& v) const { return E3MVECTOR4D(x-v.x, y-v.y, z-v.z, w-v.w); }		
	inline E3MVECTOR4D operator*(const float k)      const { return E3MVECTOR4D(x*k, y*k, z*k, w*k); }
	inline E3MVECTOR4D operator*(const E3MVECTOR4D& v) const { return E3MVECTOR4D(x*v.x, y*v.y, z*v.z, w*v.w); }	
	inline E3MVECTOR4D operator/(const float k)      const { return E3MVECTOR4D(x/k, y/k, z/k, w/k); }
	inline E3MVECTOR4D operator/(const E3MVECTOR4D& v) const { return E3MVECTOR4D(x/v.x, y/v.y, z/v.z, w/v.w); } 			
	inline float operator%(const E3MVECTOR4D& v) const { return (x*v.x + y*v.y + z*v.z + w*v.w); } // Dot product	
	inline E3MVECTOR4D operator=(const E3MVECTOR2D& v) { x=v.x; y=v.y; z=0.0f; w=1.0f; return *this; }
	inline E3MVECTOR4D operator=(const E3MVECTOR3D& v) { x=v.x; y=v.y; z=v.z; w=1.0f; return *this; }
	inline E3MVECTOR4D operator-=(const E3MVECTOR4D& v) { x-=v.x; y-=v.y; z-=v.z; w-=v.w; return *this; }
	inline E3MVECTOR4D operator+=(const E3MVECTOR4D& v) { x+=v.x; y+=v.y; z+=v.z; w+=v.w; return *this; }
	inline E3MVECTOR4D operator*=(const float k)      { x*=k; y*=k; z*=k; w*=k; return *this; }
	inline E3MVECTOR4D operator*=(const E3MVECTOR4D& v) { x*=v.x; y*=v.y; z*=v.z; w*=v.w; return *this; } 
	inline E3MVECTOR4D operator/=(const float k)      { x/=k; y/=k; z/=k; w/=k; return *this; }
	inline E3MVECTOR4D operator/=(const E3MVECTOR4D& v) { x/=v.x; y/=v.y; z/=v.z; w/=v.w; return *this; }	
	inline int operator==(const E3MVECTOR4D& v) const { return ((v.x==x)&&(v.y==y)&&(v.z==z)&&(v.w==w)); }
	inline int operator!=(const E3MVECTOR4D& v) const { return ((v.x!=x)||(v.y!=y)||(v.z!=z)||(v.w!=w)); }
	
	// Assignement:   
	inline void reset() { x=y=z=w=0.0f; }	
   inline E3MVECTOR4D set(float X, float Y, float Z, float W)     { x=X; y=Y; z=Z; w=W; return *this; }
   inline E3MVECTOR4D set(double X, double Y, double Z, double W) { x=(float)X; y=(float)Y; z=(float)Z; w=(float)W; return *this; }
   inline E3MVECTOR4D set(int X, int Y, int Z, int W)             { x=(float)X; y=(float)Y; z=(float)Z; w=(float)W; return *this; }
   inline E3MVECTOR4D set(float af[4])								    { x=af[0]; y=af[1]; z=af[2]; w=af[3]; return *this; }  
	
	// Base functions:
	inline E3MVECTOR4D lerp(E3MVECTOR4D &v, const float k) {	return (*this + ((v - *this) * k)); }
	inline float length() const 
	{	
#ifdef E3M_SAFEMODE
		return (float) sqrt(x*x+y*y+z*z+w*w); 
#else
		return sqrtf(x*x+y*y+z*z+w*w);
#endif
	}
	inline float squaredLength() const { return x*x+y*y+z*z+w*w; }	
	inline E3MVECTOR4D setLength(const float k) { register float _ratio=k/length(); x*=_ratio;	y*=_ratio; z*=_ratio; w*=_ratio; return *this; }
	inline E3MVECTOR4D normalize() { register float len=length(); if (len==0.0f) len=1.0f; x/=len; y/=len; z/=len; w/=len; return *this; }	
	
	// Advanced functions:
	inline double angle(const E3MVECTOR4D& v)	
	{
#ifdef E3M_SAFEMODE
		double dotProduct = *this % v;					
		double vectorsMagnitude = length() * v.length();
		double angle = acos(dotProduct / vectorsMagnitude);		
		if (_isnan(angle))
				return 0;			
		return(angle);
#else
		float dotProduct = *this % v;					
		float vectorsMagnitude = length() * v.length();
		float angle = acosf(dotProduct / vectorsMagnitude);		
		if (_isnan(angle))
				return 0;			
		return((double)angle);		
#endif
	}
	inline float distance(const E3MVECTOR3D& v)
	{	
		register float dx = v.x - x;
		register float dy = v.y - y;
		register float dz = v.z - z;
		register float dw = v.w - w;
#ifdef E3M_SAFEMODE
		return (float) sqrt(dx*dx + dy*dy + dz*dz + dw*dw);
#else
		return sqrtf(dx*dx + dy*dy + dz*dz + dw*dw);
#endif
	}	
	
	// Quaternions:
	inline E3MVECTOR4D quaternionSetIdentity()
	{
	   x=y=z=0.0f; w=1.0f;
	   return *this;
	}
	inline void quaternionSetEulers(float e1, float e2, float e3)
	{
	   float f1 = (float) sin(e1 * 0.5f);
      float f2 = (float) cos(e1 * 0.5f);
      float f3 = (float) sin(e2 * 0.5f);
      float f4 = (float) cos(e2 * 0.5f);
      float f5 = (float) sin(e3 * 0.5f);
      float f6 = (float) cos(e3 * 0.5f);
      x = f2 * f3 * f6 + f1 * f4 * f5;
      y = f1 * f4 * f6 - f2 * f3 * f5;
      z = f2 * f4 * f5 - f1 * f3 * f6;
      w = f2 * f4 * f6 + f1 * f3 * f5;
   }
   inline void quaternionSetEulers(E3MVECTOR3D &v)  { quaternionSetEulers(v.x, v.y, v.z); }   
   inline E3MVECTOR3D quaternionGetEulers()
   {
      E3MVECTOR3D r;
      r.x = x;
      r.y = y;
      r.z = z;
      return r;         
   }
   inline E3MVECTOR4D quaternionGetAxisAngles()
   {
      E3MVECTOR4D r;
      r.w = 2.0f * (float) acos(w);
      if (sqrt(pow(r.w, 2.0f)) < 0.0001f)
      {
            r.x = 0.0f;
            r.y = 1.0f;
            r.z = 0.0f;
            return r;
      }
      else
      {
            float g = 1.0f / (float) sin(r.w * 0.5f);
            r.x = x * g;
            r.y = y * g;
            r.z = z * g;
            return r;
      }      
   }
	inline E3MVECTOR4D quaternionInverted()
	{
		E3MVECTOR4D r;
		r.w = w;
		r.x = -x;
		r.y = -y;
		r.z = -z;
		return r;
	}	
	inline E3MVECTOR4D quaternionInvert()
	{	
		x = -x;
		y = -y;
		z = -z;
		return *this;		
	}	
	inline E3MVECTOR4D quaternionMultiply(E3MVECTOR4D q)    
	{
		E3MVECTOR4D r;
		r.w = w*q.w - x*q.x  -  y*q.y  -  z*q.z;
		r.x = w*q.x + x*q.w  +  y*q.z  -  z*q.y;
		r.y = w*q.y + y*q.w  +  z*q.x  -  x*q.z;
		r.z = w*q.z + z*q.w  +  x*q.y  -  y*q.x;
		return r;
	}
		
	// Matrix * Vector ...
	// Vector Matrix relative
} E3M_SSE E3MVECTOR4D; // 4D vector class





////////////////////////////////////////////////////////
typedef class E3M_API E3M_SSE E3MPLANE // Plane class //
////////////////////////////////////////////////////////
{
/////////////
   public: //
/////////////

	// Components:
	struct 
	{	
	   float a, b, c, d; 		
	};	
	
	// Constructors:
	inline E3MPLANE() { a=b=c=d=0.0f; }
	inline E3MPLANE(const E3MPLANE &kPlane) { a=kPlane.a; b=kPlane.b; c=kPlane.c; d=kPlane.d; }
	inline E3MPLANE(float A, float B, float C, float D) { a=A; b=B; c=C; d=D; }
	inline E3MPLANE(const E3MVECTOR3D &kNormal, float fDistance) { a=kNormal.x; b=kNormal.y; c=kNormal.z; d=fDistance; }
	inline E3MPLANE(const E3MVECTOR3D &kVector1, const E3MVECTOR3D &kVector2, const E3MVECTOR3D &kVector3)
	{
		E3MVECTOR3D kVectorA(kVector2 - kVector1);
		E3MVECTOR3D kVectorB(kVector3 - kVector1);
		E3MVECTOR3D n = (kVectorA^kVectorB).normalize();
		a = n.x;
		b = n.y;
		c = n.z;
		d = -(n%kVector1);		
	}	
	
	// Operators:
	inline E3MPLANE operator=(const E3MPLANE &kPlane) { a=kPlane.a; b=kPlane.b; c=kPlane.c; d=kPlane.d; return *this; }
	
	// Advanced functions:
	inline float distance(E3MVECTOR3D &kPoint) { return (a*kPoint.x + b*kPoint.y + c*kPoint.z + d); }
	inline float signedDistance(const E3MVECTOR3D &kPoint) const {	E3MVECTOR3D n(a, b, c); return (n%kPoint) + d; }
	inline bool isFrontFacingTo(const E3MVECTOR3D &direction) const 
	{	
	      E3MVECTOR3D n(a, b, c);		
			float dot = n%direction;
			return (dot <= 0.0f);
	}	
} E3M_SSE E3MPLANE;






//////////////////////////////////////////////////////////////
typedef class E3M_API E3M_SSE E3MMATRIX // 4x4 matrix class //
//////////////////////////////////////////////////////////////
{
/////////////
   public: //
/////////////

   // Components:
   union
	{
		float d[16];
		float m[4][4];
		struct
		{
			float _11, _12, _13, _14;	
			float _21, _22, _23, _24;
			float _31, _32, _33, _34;
			float _41, _42, _43, _44;
		};
		struct
		{		   
			float rx[4];			
			float ry[4];
			float rz[4];
			union
			{
				float t[3];				
				struct
				{
					float x, y, z;
				};
				float w;
			};
		};
	};
	
	// Constructors:
	inline E3MMATRIX() { setIdentity(); }
	inline E3MMATRIX(float *pfData)
	{
		_11 = pfData[ 0];	_12 = pfData[ 1];	_13 = pfData[ 2];	_14 = pfData[ 3];
		_21 = pfData[ 4];	_22 = pfData[ 5];	_23 = pfData[ 6];	_24 = pfData[ 7];
		_31 = pfData[ 8];	_32 = pfData[ 9];	_33 = pfData[10];	_34 = pfData[11];
		_41 = pfData[12];	_42 = pfData[13];	_43 = pfData[14];	_44 = pfData[15];
	}
	inline E3MMATRIX(float _11, float _12, float _13, float _14,
					     float _21, float _22, float _23, float _24,
					     float _31, float _32, float _33, float _34,
					     float _41, float _42, float _43, float _44)
	{
		this->_11 = _11;	this->_12 = _12;	this->_13 = _13;	this->_14 = _14;
		this->_21 = _21;	this->_22 = _22;	this->_23 = _23;	this->_24 = _24;
		this->_31 = _31;	this->_32 = _32;	this->_33 = _33;	this->_34 = _34;
		this->_41 = _41;	this->_42 = _42;	this->_43 = _43;	this->_44 = _44;
	}
	inline E3MMATRIX(const E3MMATRIX &kMatrix)
	{
		_11 = kMatrix._11;	_12 = kMatrix._12;	_13 = kMatrix._13;	_14 = kMatrix._14;
		_21 = kMatrix._21;	_22 = kMatrix._22;	_23 = kMatrix._23;	_24 = kMatrix._24;
		_31 = kMatrix._31;	_32 = kMatrix._32;	_33 = kMatrix._33;	_34 = kMatrix._34;
		_41 = kMatrix._41;	_42 = kMatrix._42;	_43 = kMatrix._43;	_44 = kMatrix._44;
	}

   // Settings:
   inline E3MVECTOR4D getRow(int position ) const	
   {
	   if (position==0)
		      return E3MVECTOR4D(d[0], d[4], d[8], d[12]);
	
	   if (position==1)
		      return E3MVECTOR4D(d[1], d[5], d[9], d[13]);
	
	   if (position==2)
		      return E3MVECTOR4D(d[2], d[6], d[10], d[14]);
	
	   if (position==3)
		      return E3MVECTOR4D(d[3], d[7], d[11], d[15]);

	   return E3MVECTOR4D(0.0f, 0.0f, 0.0f, 0.0f);
   }

   inline E3MVECTOR4D getColumn(int position) const
   {
	   if (position==0)
		      return E3MVECTOR4D(d[0], d[1], d[2], d[3]);
	
	   if (position==1)
		      return E3MVECTOR4D(d[4], d[5], d[6], d[7]);
	
	   if (position==2)
		      return E3MVECTOR4D(d[8], d[9], d[10], d[11]);
	
	   if (position==3)
		      return E3MVECTOR4D(d[12], d[13], d[14], d[15]);

	   return E3MVECTOR4D(0.0f, 0.0f, 0.0f, 0.0f);
   }
	
	// Operators:
	inline E3MMATRIX operator*(const E3MMATRIX &kMatrix)
	{
			return E3MMATRIX(((_11 * kMatrix._11) + (_12 * kMatrix._21) + (_13 * kMatrix._31) + (_14 * kMatrix._41)),
								  ((_11 * kMatrix._12) + (_12 * kMatrix._22) + (_13 * kMatrix._32) + (_14 * kMatrix._42)),
   							  ((_11 * kMatrix._13) + (_12 * kMatrix._23) + (_13 * kMatrix._33) + (_14 * kMatrix._43)),
								  ((_11 * kMatrix._14) + (_12 * kMatrix._24) + (_13 * kMatrix._34) + (_14 * kMatrix._44)),

					           ((_21 * kMatrix._11) + (_22 * kMatrix._21) + (_23 * kMatrix._31) + (_24 * kMatrix._41)),
					           ((_21 * kMatrix._12) + (_22 * kMatrix._22) + (_23 * kMatrix._32) + (_24 * kMatrix._42)),
					           ((_21 * kMatrix._13) + (_22 * kMatrix._23) + (_23 * kMatrix._33) + (_24 * kMatrix._43)),
					           ((_21 * kMatrix._14) + (_22 * kMatrix._24) + (_23 * kMatrix._34) + (_24 * kMatrix._44)),

					           ((_31 * kMatrix._11) + (_32 * kMatrix._21) + (_33 * kMatrix._31) + (_34 * kMatrix._41)),
					           ((_31 * kMatrix._12) + (_32 * kMatrix._22) + (_33 * kMatrix._32) + (_34 * kMatrix._42)),
					           ((_31 * kMatrix._13) + (_32 * kMatrix._23) + (_33 * kMatrix._33) + (_34 * kMatrix._43)),
    			              ((_31 * kMatrix._14) + (_32 * kMatrix._24) + (_33 * kMatrix._34) + (_34 * kMatrix._44)),

					           ((_41 * kMatrix._11) + (_42 * kMatrix._21) + (_43 * kMatrix._31) + (_44 * kMatrix._41)),
					           ((_41 * kMatrix._12) + (_42 * kMatrix._22) + (_43 * kMatrix._32) + (_44 * kMatrix._42)),
					           ((_41 * kMatrix._13) + (_42 * kMatrix._23) + (_43 * kMatrix._33) + (_44 * kMatrix._43)),
					           ((_41 * kMatrix._14) + (_42 * kMatrix._24) + (_43 * kMatrix._34) + (_44 * kMatrix._44)));
	}
	inline E3MMATRIX operator=(const E3MMATRIX &kMatrix)
	{
		_11 = kMatrix._11;	_12 = kMatrix._12;	_13 = kMatrix._13;	_14 = kMatrix._14;
		_21 = kMatrix._21;	_22 = kMatrix._22;	_23 = kMatrix._23;	_24 = kMatrix._24;
		_31 = kMatrix._31;	_32 = kMatrix._32;	_33 = kMatrix._33;	_34 = kMatrix._34;
		_41 = kMatrix._41;	_42 = kMatrix._42;	_43 = kMatrix._43;	_44 = kMatrix._44;
		return *this;
	}
	inline E3MMATRIX operator*=(const E3MMATRIX &kMatrix)
	{
		*this = E3MMATRIX(((_11 * kMatrix._11) + (_12 * kMatrix._21) + (_13 * kMatrix._31) + (_14 * kMatrix._41)),
							   ((_11 * kMatrix._12) + (_12 * kMatrix._22) + (_13 * kMatrix._32) + (_14 * kMatrix._42)),
					         ((_11 * kMatrix._13) + (_12 * kMatrix._23) + (_13 * kMatrix._33) + (_14 * kMatrix._43)),
					         ((_11 * kMatrix._14) + (_12 * kMatrix._24) + (_13 * kMatrix._34) + (_14 * kMatrix._44)),
					         ((_21 * kMatrix._11) + (_22 * kMatrix._21) + (_23 * kMatrix._31) + (_24 * kMatrix._41)),
					         ((_21 * kMatrix._12) + (_22 * kMatrix._22) + (_23 * kMatrix._32) + (_24 * kMatrix._42)),
					         ((_21 * kMatrix._13) + (_22 * kMatrix._23) + (_23 * kMatrix._33) + (_24 * kMatrix._43)),
					         ((_21 * kMatrix._14) + (_22 * kMatrix._24) + (_23 * kMatrix._34) + (_24 * kMatrix._44)),
					         ((_31 * kMatrix._11) + (_32 * kMatrix._21) + (_33 * kMatrix._31) + (_34 * kMatrix._41)),
					         ((_31 * kMatrix._12) + (_32 * kMatrix._22) + (_33 * kMatrix._32) + (_34 * kMatrix._42)),
					         ((_31 * kMatrix._13) + (_32 * kMatrix._23) + (_33 * kMatrix._33) + (_34 * kMatrix._43)),
					         ((_31 * kMatrix._14) + (_32 * kMatrix._24) + (_33 * kMatrix._34) + (_34 * kMatrix._44)),
					         ((_41 * kMatrix._11) + (_42 * kMatrix._21) + (_43 * kMatrix._31) + (_44 * kMatrix._41)),
					         ((_41 * kMatrix._12) + (_42 * kMatrix._22) + (_43 * kMatrix._32) + (_44 * kMatrix._42)),
					         ((_41 * kMatrix._13) + (_42 * kMatrix._23) + (_43 * kMatrix._33) + (_44 * kMatrix._43)),
					         ((_41 * kMatrix._14) + (_42 * kMatrix._24) + (_43 * kMatrix._34) + (_44 * kMatrix._44)));
		return *this;
	}	
	
	// Base functions:
	inline E3MVECTOR3D getTranslation() const
	{
	   E3MVECTOR3D _d(x, y, z);
	   return _d;
	}
	inline void setTranslation(float x, float y, float z)
	{
	   t[0] = x; t[1] = y; t[2] = z;
	}
	inline void setTranslation(E3MVECTOR3D &v)
	{
	   t[0] = v.x; t[1] = v.y; t[2] = v.z;
	}
	inline void setIdentity()
	{	
		_11 = _22 = _33 = _44 = 1.0f;
		_12 = _13 = _14 = 0.0f;
		_21 = _23 = _24 = 0.0f;
		_31 = _32 = _34 = 0.0f;
		_41 = _42 = _43 = 0.0f;
	}		
	inline void setRotationX(float angle)
	{
		angle /= 180.0f/E3M_PI;
		rx[0] = 1.0f; rx[1] = 0.0f;               rx[2] = 0.0f;              rx[3] = 0.0f;		
		ry[0] = 0.0f; ry[1] = (float) cos(angle); ry[2] = (float)sin(angle); ry[3] = 0.0f;		
		rz[0] = 0.0f; rz[1] = (float)-sin(angle); rz[2] = (float)cos(angle); rz[3] = 0.0f;
		t[0] = t[1] = t[2] = 0.0f;
		w  = 1.0f;
	}
	inline void setRotationY(float angle)
	{
		angle /= 180.0f/E3M_PI;		
		rx[0] = (float)cos(angle); rx[1] = 0.0f; rx[2] = (float)-sin(angle); rx[3] = 0.0f;
		ry[0] = 0.0f;              ry[1] = 1.0f; ry[2] = 0.0f;               ry[3] = 0.0f;		
		rz[0] = (float)sin(angle); rz[1] = 0.0f; rz[2] = (float)cos(angle);  rz[3] = 0.0f;
		t[0] = t[1] = t[2] = 0.0f;
		w  = 1.0f;
	}
	inline void setRotationZ(float angle)
	{
		angle /= 180.0f/E3M_PI;		
		rx[0] = (float)cos(angle); rx[1] = (float)sin(angle); rx[2] = 0.0f; rx[3] = 0.0f;		
		ry[0] = (float)-sin(angle); ry[1] = (float)cos(angle); ry[2] = 0.0f; ry[3] = 0.0f;		
		rz[0] = 0.0f; rz[1] = 0.0f; rz[2] = 1.0f; rz[3] = 0.0f;
		t[0] = t[1] = t[2] = 0.0f;
		w  = 1.0f;
	}
	inline void setRotation(float angle, float x, float y, float z)
	{
	   const float a = E3M_TORAD(angle);
	   const float c = (float) cos(a);
	   const float s = (float) sin(a);
	   const float oneMinusC = 1.0f-c;	
	   const float zMulOneMinusC = z*oneMinusC;
	   const float xMulOneMinusC = x*oneMinusC;
	   const float yMulOneMinusC = y*oneMinusC;	   
	   d[0] = x*xMulOneMinusC+c;   d[1] = x*yMulOneMinusC+z*s; d[2] = x*zMulOneMinusC-y*s; d[3]  = 0.0f;
	   d[4] = y*xMulOneMinusC-z*s; d[5] = y*yMulOneMinusC+c;   d[6] = y*zMulOneMinusC+x*s; d[7]  = 0.0f;
	   d[8] = x*zMulOneMinusC+y*s; d[9] = y*zMulOneMinusC-x*s; d[10]= z*zMulOneMinusC+c;   d[11] = 0.0f;
	   d[12] = 0.0f;               d[13]= 0.0f;                d[14]= 0.0f;                d[15] = 1.0f;	   
	}
   inline void setScaling(const E3MVECTOR3D &kScale)
	{
		_11 = kScale.x;
		_22 = kScale.y;
		_33 = kScale.z;
	}
	inline void transpose()
	{
		E3MMATRIX kMatrix(*this);
		_11 = kMatrix._11;    _12 = kMatrix._21;    _13 = kMatrix._31;    _14 = kMatrix._41;
		_21 = kMatrix._12;    _22 = kMatrix._22;    _23 = kMatrix._32;    _24 = kMatrix._42;
		_31 = kMatrix._13;    _32 = kMatrix._23;    _33 = kMatrix._33;    _34 = kMatrix._43;
		_41 = kMatrix._14;    _42 = kMatrix._24;    _43 = kMatrix._34;    _44 = kMatrix._44;		
	}	
	inline E3MMATRIX inverse()
	{
		E3MMATRIX kResult;
		float fDetInverse = (1.0f / determinant());
		kResult._11 = ( fDetInverse * ((_22 * _33) - (_23 * _32)));
		kResult._12 = (-fDetInverse * ((_12 * _33) - (_13 * _32)));
		kResult._13 = ( fDetInverse * ((_12 * _23) - (_13 * _22)));
		kResult._14 = 0.0f;

		kResult._21 = (-fDetInverse * ((_21 * _33) - (_23 * _31)));
		kResult._22 = ( fDetInverse * ((_11 * _33) - (_13 * _31)));
		kResult._23 = (-fDetInverse * ((_11 * _23) - (_13 * _21)));
		kResult._24 = 0.0f;

		kResult._31 = ( fDetInverse * ((_21 * _32) - (_22 * _31)));
		kResult._32 = (-fDetInverse * ((_11 * _32) - (_12 * _31)));
		kResult._33 = ( fDetInverse * ((_11 * _22) - (_12 * _21)));
		kResult._34 = 0.0f;

		kResult._41 = -((_41 * kResult._11) + (_42 * kResult._21) + (_43 * kResult._31));
		kResult._42 = -((_41 * kResult._12) + (_42 * kResult._22) + (_43 * kResult._32));
		kResult._43 = -((_41 * kResult._13) + (_42 * kResult._23) + (_43 * kResult._33));
		kResult._44 = 1.0f;
		return kResult;
	}
	inline void invert()
	{
		*this = inverse();
	}
	inline float determinant()
	{
		return ((_11 * ((_22 * _33) - (_23 * _32))) -
              (_12 * ((_21 * _33) - (_23 * _31))) +
              (_13 * ((_21 * _32) - (_22 * _31))));
	}
	
	// Advanced functions:
	inline E3MVECTOR3D rotate(const E3MVECTOR2D &kVector)
	{
		return E3MVECTOR3D(((kVector.x * _11) + (kVector.y * _21)),
				     		    ((kVector.x * _12) + (kVector.y * _22)),
					          ((kVector.x * _13) + (kVector.y * _23)));
	}
	inline E3MVECTOR3D rotate(const E3MVECTOR3D &kVector)
	{
		return E3MVECTOR3D(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31)),
								 ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32)),
								 ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33)));
	}
	inline E3MVECTOR3D rotate(const E3MVECTOR4D &kVector)
	{
		return E3MVECTOR3D(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31)),
								 ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32)),
								 ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33)));
	}
	inline E3MVECTOR4D transform(const E3MVECTOR2D &kVector) const
	{
		return E3MVECTOR4D(((kVector.x * _11) + (kVector.y * _21) + _41),
								 ((kVector.x * _12) + (kVector.y * _22) + _42),
								 ((kVector.x * _13) + (kVector.y * _23) + _43),
								 ((kVector.x * _14) + (kVector.y * _24) + _44));
	}
	inline E3MVECTOR4D transform(const E3MVECTOR3D &kVector) const
	{
		return E3MVECTOR4D(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31) + _41),
								 ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32) + _42),
								 ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33) + _43),
								 ((kVector.x * _14) + (kVector.y * _24) + (kVector.z * _34) + _44));
	}
	inline E3MVECTOR4D transform(const E3MVECTOR4D &kVector) const
	{
		return E3MVECTOR4D(((kVector.x * _11) + (kVector.y * _21) + (kVector.z * _31) + (kVector.w * _41)),
								 ((kVector.x * _12) + (kVector.y * _22) + (kVector.z * _32) + (kVector.w * _42)),
								 ((kVector.x * _13) + (kVector.y * _23) + (kVector.z * _33) + (kVector.w * _43)),
								 ((kVector.x * _14) + (kVector.y * _24) + (kVector.z * _34) + (kVector.w * _44)));
	}
	
	// Quaternions:
	inline E3MMATRIX fromQuaternion(E3MVECTOR4D q)
	{		
		float xx = q.x * q.x, xy = q.x * q.y, xz = q.x * q.z, xw = q.x * q.w;
		float yy = q.y * q.y, yz = q.y * q.z, yw = q.y * q.w;
		float zz = q.z * q.z, zw = q.z * q.w;

		d[ 0] = 1.0f - 2.0f * ( yy + zz );
		d[ 1] = 2.0f * ( xy - zw );  
		d[ 2] = 2.0f * ( xz + yw );  
		d[ 3] = 0.0f;  

		d[ 4] = 2.0f * ( xy +  zw );  
		d[ 5] = 1.0f - 2.0f * ( xx + zz );  
		d[ 6] = 2.0f * ( yz - xw );  
		d[ 7] = 0.0f;  

		d[ 8] = 2.0f * ( xz - yw );  
		d[ 9] = 2.0f * ( yz + xw );  
		d[10] = 1.0f - 2.0f * ( xx + yy );  
		d[11] = 0.0f;  

		d[12] = 0.0f;  
		d[13] = 0.0f;   
		d[14] = 0.0f;   
		d[15] = 1.0f;
		
		return *this;
	}		
	inline E3MVECTOR4D toQuaterion() 
	{
	   E3MVECTOR4D q;
      float trace = m[0][0] + m[1][1] + m[2][2] + 1.0f;      
      
      if (trace > 0.0f) 
      {
            float s = 0.5f / sqrtf(trace);
            q.w = 0.25f / s;
            q.x = (m[2][1] - m[1][2]) * s;
            q.y = (m[0][2] - m[2][0]) * s;
            q.z = (m[1][0] - m[0][1]) * s;
      } 
      else 
      {
            if (m[0][0]>m[1][1] && m[0][0]>m[2][2]) 
            {
                  float s = 2.0f * sqrtf(1.0f + m[0][0] - m[1][1] - m[2][2]);
                  q.x = 0.25f * s;
                  q.y = (m[0][1] + m[1][0]) / s;
                  q.z = (m[0][2] + m[2][0]) / s;
                  q.w = (m[1][2] - m[2][1]) / s;
            } 
            else 
                  if (m[1][1] > m[2][2]) 
                  {
                        float s = 2.0f * sqrtf(1.0f + m[1][1] - m[0][0] - m[2][2]);
                        q.x = (m[0][1] + m[1][0]) / s;
                        q.y = 0.25f * s;
                        q.z = (m[1][2] + m[2][1]) / s;
                        q.w = (m[0][2] - m[2][0]) / s;    
                   } 
                   else 
                   {                      
                        float s = 2.0f * sqrtf(1.0f + m[2][2] - m[0][0] - m[1][1]);
                        q.x = (m[0][2] + m[2][0]) / s;
                        q.y = (m[1][2] + m[2][1]) / s;
                        q.z = 0.25f * s;
                        q.w = (m[0][1] - m[1][0]) / s;
                  }
      }
      
      return q;
   }
   inline E3MVECTOR4D toQuaternionJona()
   {
      E3MVECTOR4D outV;
      float  tr, s, q[4];
      int    i, j, k;

      int nxt[3] = {1, 2, 0};
      tr = m[0][0] + m[1][1] + m[2][2];

      // check the diagonal
      if (tr > 0.0f)
      {
            s = sqrtf(tr + 1.0f);
            outV.w = s / 2.0f;
            s = 0.5f / s;
            outV.x = (m[1][2] - m[2][1]) * s;
            outV.y = (m[2][0] - m[0][2]) * s;
            outV.z = (m[0][1] - m[1][0]) * s;
      }
      else
      {           // diagonal is negative
            i = 0;
            if (m[1][1] > m[0][0]) i = 1;
            if (m[2][2] > m[i][i]) i = 2;
            j = nxt[i];
            k = nxt[j];
   
            s = sqrtf ((m[i][i] - (m[j][j] + m[k][k])) + 1.0f);
            q[i] = s * 0.5f;
            if (s != 0.0f) s = 0.5f / s;

            q[3] = (m[j][k] - m[k][j]) * s;
            q[j] = (m[i][j] + m[j][i]) * s;
            q[k] = (m[i][k] + m[k][i]) * s;

            outV.x = q[0];
            outV.y = q[1];
            outV.z = q[2];
            outV.w = q[3];
      }
      return outV;
   }  
   
   // Euler angles:
   inline E3MVECTOR3D toEuler() 
   {
      E3MVECTOR3D output;
      // Assuming the angles are in radians.
	   if (m[1][0] > 0.998) 
	   { 
	         // singularity at north pole
		      output.x = (float) E3M_TODEC(atan2(m[0][2], m[2][2]));		
		      output.y = (float) E3M_TODEC(E3M_PI/2.0);
		      output.z = (float) E3M_TODEC(0.0f);
		      return output;
	   }	
	   if (m[1][0] < -0.998) 
	   { 
	         // singularity at south pole
		      output.x = (float) E3M_TODEC(atan2(m[0][2], m[2][2]));		
		      output.y = (float) E3M_TODEC(-E3M_PI/2.0);
		      output.z = (float) E3M_TODEC(0.0f);
		      return output;
	   }	
	   output.x = (float) E3M_TODEC(atan2(-m[2][0], m[0][0]));
	   output.y = (float) E3M_TODEC(atan2(-m[1][2], m[1][1]));
	   output.z = (float) E3M_TODEC(asin(m[1][0]));
	   return output;
   }

} E3M_SSE E3MMATRIX;



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
