//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//																																									 //
// Project: Mental Connection Client																													 //
// Author : Achille Peternier, VRLab, EPFL (C) 2005-2007																							 //
// Release: 0.4a, 17/6/2005																																 //	
//																																									 //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _MCLIENT_H
#define _MCLIENT_H



//////////////
// #INCLUDE //
//////////////
#if defined(WIN32) || defined(UNDER_CE)
   #include <winsock2.h>
#else // LINUX
   #include <sys/types.h>
   #include <sys/socket.h>   
   #include <sys/stat.h>
   #include <netinet/tcp.h>
   #include <netdb.h>   
   #include <arpa/inet.h>
   #include <string.h>
   #include <unistd.h>
   #include <dirent.h>
#endif
#include <stdio.h>
#include <time.h>
#include <mserver_messages.h>
#ifdef MC_PDA
   #include <winioctl.h>
   #include <ntddndis.h> 
   #include <nuiouser.h>
   #include <Iphlpapi.h>
#endif



/////////////
// #DEFINE //
/////////////
	
	// General:	
   #define MC_NAME					"MClient Mental Connection client v0.4a"	        
	#define MC_RELEASE				0.4
	#define MC_IPSTRINGSIZE       17
	
	// Default:
	#define MC_DEFAULT_PORT			4444
	#define MC_DEFAULT_TIMEOUT    0
	#define MC_DEFAULT_PATH       ""
	
	// Limits:
	#define MC_MAXNAMELENGTH		64
	
	// Data related:
	#define MC_BUFFERSIZE			MC_MAXMESSAGESIZE*8	
	
	// Status:
	typedef enum MC_STATUS
	{
		MC_DOWN = 0,		
		MC_DISCONNECTED, 
		MC_CONNECTED,		
	} MC_STATUS;		
   
#if !defined(WIN32) && !defined(_WIN32_CE)
   // Aux defines for Linux:
   #define MAX_PATH NAME_MAX
   #define SOCKET int
   #define SOCKET_ERROR -1
   #define INVALID_SOCKET ~0
   #define closesocket close
   #define SD_BOTH SHUT_RDWR
#endif	

#ifdef MC_PDA	
	// Types:
	#define MC_MAXNUMBEROFACCESSPOINTS 32
	typedef struct MC_ACCESSPOINT
	{
			UCHAR macAddress[6];
			LONG signal;
	} MC_ACCESSPOINT;

	// Min/max RSSI:
	#define MC_MIN_RSSI 99999999
	#define MC_MAX_RSSI -99999999
#endif		

   // Specifies i/o linkage (VC++ spec):
#if defined(WIN32) || defined(_WIN32_WCE)	
   #ifdef MCLIENT_EXPORTS
		#define MC_API __declspec(dllexport)
   #else
		#define MC_API __declspec(dllimport)
   #endif   
#else // LINUX
   #define MC_API
#endif      
   
   // Debug:    
   #ifdef _DEBUG
		#define MC_DEBUG
		#define MC_LOG(...)         exfLog(__VA_ARGS__)
		#define MC_LOGDETAILED(...) exfLog(__VA_ARGS__)
		#define MC_LOGMANIAC(...)   exfLog(__VA_ARGS__)		
	#else
		#define MC_LOG(...)         
		#define MC_LOGDETAILED(...) 
		#define MC_LOGMANIAC(...)   		
	#endif	
	
#ifdef MC_PDA			
////////////////////
// REGISTER TOOLS //
////////////////////

	BOOL RegistrySetDwordCU(LPCWSTR key, LPCWSTR name, DWORD value);
	DWORD RegistryGetDwordCU(LPCWSTR key, LPCWSTR name, DWORD defaultValue);
	BOOL RegistrySetStringCU(LPCWSTR key, LPCWSTR name, TCHAR *value);
	BOOL RegistryGetStringCU(LPCWSTR key, LPCWSTR name, ULONG bufLen, TCHAR *buf, TCHAR *defaultValue);
#endif



///////////////////
// CLASS MCLIENT //
///////////////////
typedef class MC_API MCLIENT
{
/////////////
	public: //
/////////////

	// Const/dest:
	MCLIENT();
	~MCLIENT();
	
	// Init/free:
	bool init();
	bool free();
	bool connectToServer(int _port, char *_IP);
	bool disconnectFromServer();	
	bool login(char *username, char *password);
	
	// Settings:
	inline void setTimeOut(unsigned t)        { timeOut = (int) t; }    
	inline void setDownloadPath(char *path)   { if (path) strcpy(downloadPath, path); }
	inline MC_STATUS getStatus()              { return status; }
	inline int getTimeOut()                   { return timeOut; }
	inline char *getDownloadPath()            { return downloadPath; }
	unsigned getInactivityTime();       
	bool NagleOn();
	bool NagleOff();
	
	// Data handling:
	bool getDataFromStream();
	bool sendData(unsigned short size, char *data); 	
	bool sendFile(char *filename);
	bool sendDirectory(char *directory);
	
	// Messages handling:
	bool ping();
	
	// Kernel:
	bool acquireData();
	bool manageMessages();	
	inline void setCustomParser(void *myFunc) { parserFunc = myFunc; }
	
#ifdef MC_PDA	
	// Positioning (NDIS):
	static bool initAdapter();
	static bool releaseAdapter();
	static bool resetCalibration();
	static bool isAdapterActive()									{ return deviceActive; }
	static int scanAPs();
	static int getCurrentAPSignalStrength();
	static bool getCurrentAPMacAddress(UCHAR MacAddress[6]);
	static inline UCHAR *getScannedAPMacAddress(int c)		{ return accessPoint[c].macAddress; }
	static inline LONG getScannedAPSignalStrength(int c)  { return accessPoint[c].signal; }
	static inline int getMaxSignalStrength() { return maxRSSI; }
	static inline int getMinSignalStrength() { return minRSSI; }
#endif	
	
//////////////
	private: //
//////////////

	// Data:
	int port;
	SOCKET clientSocket;
	sockaddr_in address;
	char IP[MC_IPSTRINGSIZE];
	char name[MC_MAXNAMELENGTH];
		
	// Status:
	MC_STATUS status;		
	
	// Timer:
	int timeOut;
	clock_t timer;
	
	// Buffer:
	unsigned bufferPosition;
	char buffer[MC_BUFFERSIZE];
	
	// Custom parser:
	void *parserFunc;
	
	// Download:
	bool fileTransfer;
	FILE *DAT;
	char downloadPath[MAX_PATH];
	
#ifdef MC_PDA  
	// Positioning:
	static TCHAR currentDeviceName[MAX_ADAPTER_NAME_LENGTH + 4];
	static bool deviceActive;
	static HANDLE openedDevice;
	static MC_ACCESSPOINT accessPoint[MC_MAXNUMBEROFACCESSPOINTS];
	static int minRSSI;
	static int maxRSSI;

	// Keep private:
	static HANDLE openDevice(LPCTSTR lpDeviceName);	
	static int getDeviceSignalStrength(TCHAR* wName);		
	static TCHAR *getCurrentDevice() { return currentDeviceName; }	
#endif	
} MCLIENT;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
