//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//																																									 //
// Project: Mental Connection Server																													 //
// Author : Achille Peternier, VRLab, EPFL (C) 2005-2007																							 //
// Release: 0.4a, 16/6/2005																																 //	
//																																									 //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _MSERVER_H
#define _MSERVER_H



//////////////
// #INCLUDE //
//////////////
#if defined(WIN32) || defined(UNDER_CE)
   #include <winsock2.h>
#else // LINUX
   #include <sys/types.h>
   #include <sys/socket.h>   
   #include <sys/stat.h>
   #include <netinet/tcp.h>
   #include <netdb.h>   
   #include <arpa/inet.h>
   #include <string.h>
   #include <unistd.h>
   #include <dirent.h>
#endif
#include <time.h>



/////////////
// #DEFINE //
/////////////

	// General:	
	#define MS_NAME								"MServer Mental Connection server v0.4a"
	#define MS_RELEASE							0.4
	#define MS_IPSTRINGSIZE						17	
	
	// Default:
	#define MS_DEFAULT_PORT						4444
	#define MS_DEFAULT_SLEEPTIME				25
	#define MS_DEFAULT_MAXNUMBEROFCLIENTS	32	
	#define MS_DEFAULT_TIMEOUT					0 
	#define MS_DEFAULT_PINGTIMEOUT         0
	#define MS_DEFAULT_MULTIIP					true
	#define MS_DEFAULT_USERLIST            "[none]"
	
	
#if !defined(WIN32) && !defined(_WIN32_CE)
   // Aux defines for Linux:
   #define MAX_PATH NAME_MAX
   #define SOCKET int
   #define SOCKET_ERROR -1
   #define INVALID_SOCKET ~0
   #define closesocket close
   #define SD_BOTH SHUT_RDWR
#endif
	
	// Specifies i/o linkage (VC++ spec):
#if defined(WIN32) || defined(_WIN32_WCE)	
   #ifdef MSERVER_EXPORTS
		#define MS_API __declspec(dllexport)
   #else
		#define MS_API __declspec(dllimport)
   #endif   
#else // LINUX
   #define MS_API
#endif   
   
   // Debug:       
   #ifdef _DEBUG
		#define MS_DEBUG
		#define MS_LOG(...)         exfLog(__VA_ARGS__)
		#define MS_LOGDETAILED(...) exfLog(__VA_ARGS__)
		#define MS_LOGMANIAC(...)   /*exfLog(__VA_ARGS__)*/
	#else
	   #define MS_LOG(...)	      
		#define MS_LOGDETAILED(...)
		#define MS_LOGMANIAC(...)	 		
	#endif		
	
	
	
/////////////////
// SUB-HEADERS //
/////////////////	

   // Sub-include:
   #include "mserver_messages.h"
	#include "mserver_client.h"	
	
	// User list:	
	typedef struct MSUSER
	{
	      char username[MC_USERNAMELENGTH];
	      char password[MC_PASSWORDLENGTH];
	} MSUSER;

		
	
///////////////////
// CLASS MSERVER //
///////////////////
typedef class MS_API MSERVER
{
/////////////
	public: //
/////////////

	// Const/dest:
	MSERVER();
	~MSERVER();
	
	// Init/free:
	bool init(char *filename);
	bool free();
	bool loadUserList(char *filename);
	
	// Settings:
	inline void setPort(int port)                                { this->port = port; }
	inline void setIP(char *IPaddr)                              { if (IPaddr) strcpy(IP, IPaddr); else strcpy(IP, "[none]"); }
	inline void setMaxNumberOfClients(int maxNumberOfClients)    { this->maxNumberOfClients = maxNumberOfClients; }
	inline void setSleepTime(int sleepTime)                      { this->sleepTime = sleepTime; }
	inline void setTimeOut(int timeOut)                          { this->timeOut = timeOut; }
	inline void setMultiIP(bool multiIP)                         { this->multiIP = multiIP; }
	inline void setUserList(char *u)                             { if (u) strcpy(userList, u); }
	inline int getPort() const                                   { return port; }
	inline char *getIP()                                         { return IP; }  
	inline int getMaxNumberOfClients()                           { return maxNumberOfClients; }
	inline int getSleepTime()                                    { return sleepTime; }
	inline int getTimeOut()                                      { return timeOut; }
	inline bool getMultiIP()                                     { return multiIP; }	
	inline char *getUserList()                                   { return userList; }
	
	// Kernel:	
	inline void sleep()	                                        
	{
	   #if defined(WIN32) || defined(_WIN32_WCE)	
         Sleep(sleepTime); 
      #else
         usleep(sleepTime * 1000);
      #endif         
	}	
	bool NagleOn();
	bool NagleOff();
	bool acceptClients();
	bool acquireData();
	bool sendFile(char *filename);
	bool sendDirectory(char *directory);
	bool manageClients();
	inline void setCustomParser(void *myFunc)                    { parserFunc = myFunc; }
	
//////////////
	private: //
//////////////

	// Server params:
	int port, sleepTime, timeOut, pingTimeOut, maxNumberOfClients;	
	bool multiIP;	
	SOCKET serverSocket;
	sockaddr_in address;
	char IP[MS_IPSTRINGSIZE];
	int numberOfUsers;
	char userList[MAX_PATH];
	MSUSER *userDB;
	
	// Timer:
	clock_t timer;
	
	// Parser func:
	void *parserFunc;
	
	// File transfer:
	bool fileTransfer;
} MSERVER;
	


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
