//////////////////////////////////////////////
//                                          //
// project  : MServer v0.4a                 //
// file     : mserver_client.H	           //
// author   : Achille Peternier             // 
// copyright: VRLab, EPFL (C) 2005-2007     //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Default:
	#define MS_BUFFERSIZE MC_MAXMESSAGESIZE*8
	#define MS_MAXNAMELENGTH 64
	
	// Destructor proto:
	class MSERVERCLIENT;
	typedef void (*MS_CONSTRUCTORPROTO)(MSERVERCLIENT *);
	typedef void (*MS_DESTRUCTORPROTO)(void *);
	
	
	
///////////////////
// CLASS MCLIENT //
///////////////////
typedef class MS_API MSERVERCLIENT
{
	friend class MSERVER;
	
/////////////
	public: //
/////////////

	// Const/dest:
	MSERVERCLIENT();
	~MSERVERCLIENT();
	static void setCustomConstructor(void *d)     { customConstructor = d; }
	static void *getCustomConstructor()           { return customConstructor; }
	void setCustomDestructor(void *d)      { customDestructor = d; }
	void *getCustomDestructor()            { return customDestructor; }
	void setCustomData(void *d)            { customData = d; }
	void *getCustomData()                  { return customData; }
	
	// Settings:
	inline unsigned int getId() const	   { return id; }
	inline char *getName()					   { return name; }
	inline char *getIP()						   { return IP; }
	inline bool pinged()                   { if (ping) { ping = false; return true; } return false; }
	inline bool setName(char *_name)		   { if (_name && strlen(_name)<MS_MAXNAMELENGTH) { strcpy(name, _name); return true; } return false; }
	inline bool isThrusted()               { return thrusted; }
	
	// Management:
	static MSERVERCLIENT *findClientByIP(char *_IP);
   static MSERVERCLIENT *getClient(unsigned clientNr);
	inline void incReferences()			   { references++; }
	inline void decReferences()			   { references--; }
	inline int getReferences() const		   { return references; }
	static unsigned cleanUp();
	static unsigned getNumberOfClients();
	
	// Data handling:
	bool getDataFromStream();
	bool sendData(unsigned short size, char *data);	
	static bool sendDataToAll(unsigned short size, char *data, MSERVERCLIENT *self = NULL);
	
//////////////
	private: //
//////////////

	// Lists:
	static MSERVERCLIENT *firstClient;
	MSERVERCLIENT *prev, *next;

	// Data:
	unsigned int id;
	char name[MS_MAXNAMELENGTH+1];
	int references;
	SOCKET clientSocket;
	sockaddr_in address;
	char IP[MS_IPSTRINGSIZE];	
	void *customDestructor, *customData;
	bool thrusted;
	static void *customConstructor;
	
	// Buffer:
	unsigned bufferPosition;
	char buffer[MS_BUFFERSIZE];
	
	// Timer:
	clock_t timer, pingTimer;
	bool ping;
} MSERVERCLIENT;
