////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	   																																						  //
// Project: Mental Connection messages codes 																									  //
// Author : Achille Peternier, VRLab, EPFL (C) 2005-2007																						  //
// Release: 0.4a, 16/6/2005																															  //	
//																																								  //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _MSERVER_MESSAGES_H
#define _MSERVER_MESSAGES_H

//
// This file contains command codes for MConnection server/client messages handling
//
// Message protocol is:
//
// bytes   |   0        1    |    2    |  3 ... x |
// part    |            header         |   body   |
// details | ushort msg size | command |   data   |
// 
// Size is the size of the entire message without the initial 3 bytes header (that is the size of the body part).
//
// When you use sendData(A, B), A is the whole size of the buffer B you want to send, including the header. But the first
// two bytes of your message will contain A-3, that is the size of the body part only, without the header.
//


#define MC_MAXMESSAGESIZE 32768/4

typedef enum MC_COMMAND // This list must be continous
{
   MC_COM_PING = 0,   
   MC_COM_FORWARDALL,
   MC_COM_FORWARDWITHOUTSENDER, 
   MC_COM_STARTFILETRANSFER,
   MC_COM_FILETRANSFER,
   MC_COM_ENDFILETRANSFER,
   MC_COM_WAIT,
   MC_COM_SERVERFULL,   
   MC_COM_SERVERONLY,
   MC_COM_INVALIDLOGIN,
   MC_COM_LOGIN,
   MC_COM_NUMBEROFCOMMANDS,
} MC_COMMAND;

static char MCcommandLabel[MC_COM_NUMBEROFCOMMANDS][64] = 
{ 
	"ping",	
	"forward to all",
	"forward to all (except sender)",
	"start file transfer",
	"file transfer",
	"end file transfer",
	"wait",
	"server full",
	"server only",
	"invalid login",
	"login",
};

#define MC_COMMANDSTRING(x) MCcommandLabel[x]
#define MC_USERNAMELENGTH 16
#define MC_PASSWORDLENGTH 16

// Parser proto:
typedef bool (*MC_PARSERPROTO)(unsigned short size, char *data, void *sender);



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
