/////////////////////////////////////////////////////////////// 
//																			    //
//    MVisio Mental Visio 2D/3D Graphics Engine              //
//    version  : 1.6.6a													 //
//    author   : Achille Peternier                           //
//    contact  : achille.peternier@epfl.ch						 //
//    homepage : http://vrlab.epfl.ch/~apeternier            //
//    created  : 19/01/2005											 //																					
//    copyright: VRLab/EPFL (C) 2005-2009                    //
//																				 //
///////////////////////////////////////////////////////////////
#ifndef __MVISIO_ENGINE__
#define __MVISIO_ENGINE__



/////////////
// VERSION //
/////////////

	// Define this to do a PDA build:
   //#define MV_PDA			   // Define that in the project prepocessor settings (better)
	//#define MV_PDA_SOFTWARE	// Define that to enable software rendering (Gerbera OpenGL ES CL 1.0)
	//#define MV_PDA_AXIM      // Define that to build a version for Dell Axim x50v only (hardware accelerated)
	
	// Define this to do a CAVE build:
	//#define MV_CAVE          // Define that to create a network-based CAVE server of MVisio
	
	// Specifies i/o linkage (VC++ spec):
#if defined(WIN32) || defined(_WIN32_WCE)
   #ifdef MVISIO_EXPORTS
		#define MV_API __declspec(dllexport)
   #else
		#define MV_API __declspec(dllimport)
   #endif   
#else // LINUX:
   #define MV_API
#endif   



//////////////
// #INCLUDE //
//////////////

	// Headers:		
#ifdef MV_CAVE      
   #include "mclient.h"
   #include "mserver.h"   
#endif
	#include "e3dmath.h"
	#include <stdio.h>
	#include <string.h>
#if defined(WIN32) || defined(UNDER_CE)
	#include <windows.h>		
#else // LINUX:
#endif

   // #define MV_DEBUG_NOREMOTE        // Hard removal of cooperative functionalities	
	#define MV_DEBUG_NOSOUND         // Hard removal of audio interface if defined
	#ifndef MV_DEBUG_NOSOUND
	   #include <exaudio.h>
	#endif
#ifndef MV_PDA
	#include <GL/gl.h>	  
	#include "glext.h"				   // from http://oss.sgi.com/projects/ogl-sample/ABI/		
#else
	#ifdef MV_PDA_SOFTWARE
		#include "GLESsoftware/gl.h"
		#include "GLESsoftware/egl.h"		
	#endif
	#ifdef MV_PDA_AXIM
	   #include "GLESaxim/gl.h"
		#include "GLESaxim/egl.h"				
	#endif				 
#endif	
#include "glWrapper.h"  



/////////////
// #DEFINE //
/////////////

   // Debug:    
   #ifdef _DEBUG
		#define MV_DEBUG						
		#define MV_LOG(...)         exfLog(__VA_ARGS__)
		#define MV_LOGDETAILED(...) //exfLog(__VA_ARGS__)
		#define MV_LOGMANIAC(...)   //exfLog(__VA_ARGS__)
	#else	   		
		#define MV_LOG(...)
		#define MV_LOGDETAILED(...)
		#define MV_LOGMANIAC(...)	 
	#endif	

	// General:
	#if defined(MV_CAVE)
	   #define MV_PLATFORM "for CAVE"
	#elif defined(MV_PDA)
	   #ifdef MV_PDA_SOFTWARE
	      #define MV_PLATFORM "for PDA"
	   #else	   
	      #define MV_PLATFORM "for Axim"
	   #endif	   
	#else
	   #ifdef WIN32
	      #define MV_PLATFORM "for PC"
	   #else
	      #define MV_PLATFORM "for Linux"
	   #endif
	#endif	
	#ifdef MV_DEBUG
	   #define MV_NAME "MVisio 3D Graphics Engine "MV_PLATFORM" v1.6.6a (debug)"      
	#else
	   #define MV_NAME "MVisio 3D Graphics Engine "MV_PLATFORM" v1.6.6a"
	#endif
	#define MV_VERSION 1.6
	#define MV_SUBVERSION 6
	
	// Limits:	
	#define MV_MAXNUMBEROFCHARS 256
	#define MV_VALIDSTRING(x) (strlen(x) < MV_MAXNUMBEROFCHARS)
	
	// XVE related:
	#define MV_MVEFILEHEADER "MVE file v1.6a"	
	#define MV_MVAFILEHEADER "MVA file v1.6a"  
   
   // Maths:
   #define MV_PI E3M_PI   
   
   // 16 bits color masks:
   #define MV_16BITREDMASK   0xF800
   #define MV_16BITGREENMASK 0x7E0
	#define MV_16BITBLUEMASK  0x1F

	// Primitive type:
#ifdef MV_PDA
	#define NATIVE_TYPE GL_FIXED
	#define NATIVE_TYPE_CAST int
#else
	#define NATIVE_TYPE GL_FLOAT
	#define NATIVE_TYPE_CAST float
#endif   
   

   
///////////
// TYPES //
///////////	

	// Basetype:
	typedef unsigned char MVBYTE;
	typedef unsigned short int MVWORD;
	typedef unsigned int MVDWORD;
	
	// Specials:
	typedef unsigned int MVID;
	typedef E3MVECTOR2D MVUV;
	typedef struct { MVWORD a, b, c; } MVFACE; 		

	// Colors:	
	typedef E3MVECTOR3D MVCOLOR3;
	typedef E3MVECTOR4D MVCOLOR4;
	
	// Vectors:
	typedef E3MVECTOR2D MVVECTOR2;
	typedef E3MVECTOR3D MVVECTOR3;
	typedef E3MVECTOR4D MVVECTOR4;
	
	// Planes, matrices:
	typedef E3MPLANE	  MVPLANE;
	typedef E3MMATRIX   MVMATRIX;
	
 

////////////////
// SUBHEADERS //
////////////////

	// Subsystem/setting:
#ifdef MV_CAVE
   #include "mvisio_client.h"
#endif
	#include "mvisio_settings.h"		

	// Matlib and tools:
	#include "mvisio_shader.h"
	#include "mvisio_texture.h"
	#include "mvisio_material.h"
	#include "mvisio_fbo.h"
	#include "mvisio_opengl.h"	

	// Objects:	
	class MVELEMENT; // predefine
	#include "mvisio_node.h"
	#include "mvisio_camera.h"
	#include "mvisio_light.h"
	#include "mvisio_mesh.h"
	#include "mvisio_emitter.h"	
	#include "mvisio_helper.h"
#if !defined(MV_PDA) && !defined(MV_PDA_AXIM)
	#include "mvisio_skydome.h"
	#include "mvisio_water.h"
	#include "mvisio_terrain.h"
#endif
	
	// Animators:
	#include "mvisio_anim.h"
	
	// 2d:  
	#include "mvisio_box.h"
	#include "mvisio_font.h"
	#include "mvisio_label.h"
	#include "mvisio_button.h"	
	#include "mvisio_textfield.h"
	#include "mvisio_hscrollbar.h"
	#include "mvisio_vscrollbar.h"
	#include "mvisio_window.h"
	
	// Video support:
	#include "mvisio_video.h"

	// Rendering:
	#include "mvisio_element.h"	
	
	  	   

//////////////////
// CLASS MVISIO //
//////////////////

////////////////////////////////////////////
typedef class MV_API MVISIO // Main class //
////////////////////////////////////////////
{
/////////////
	public: //
/////////////	
	
	// Init/free:
	static inline bool isActive()								      { return active; };
	static bool init(MVSETTINGS *config = NULL);
	static bool free();	

	// TimeStamp related:
	static float getFPS();								     
	static float getFPSfactor();						     
	static inline long getTimeStamp()	                     { return timeStamp; }					     
	
	// Memory settings:
	static inline unsigned getVideoMemory()				      { return videoMemoryTotal; }
	static inline unsigned getFreeVideoMemory()			      { return videoMemoryTotal-videoMemoryUsed; }
	static inline unsigned getUsedVideoMemory()			      { return videoMemoryUsed; }	
	static inline unsigned pushVideoMemory(unsigned m)	      { videoMemoryUsed += m; return videoMemoryUsed; } // private?
	static inline unsigned popVideoMemory(unsigned m)	      { videoMemoryUsed -= m; return videoMemoryUsed; } // private?

	// Loading:
	static void setMVEPath(char *path)								{ if (path) strcpy(mvePath, path); else strcpy(mvePath, ""); }
	static char *getMVEPath() 											{ return mvePath; }
	static MVNODE *load(char *filename);
	static MVNODE *loadXML(char *filename);
	static MVNODE *recursiveLoad(char *buffer, long *position, MVNODE *parentNode);
	static bool recursiveInitBones(MVNODE *root, MVNODE *current);
	static MVNODE *recursiveLoadXML(MVNODE *root, void *n);
	
	// Rendering:		
	static inline void setSkybox(MVNODE *sky)                
	{
	      #ifdef MV_CAVE
	         if (sky)
	               MVSETTINGS::sendCommand(MV_SETSKYBOX, sky->getId());
            else	               
                  MVSETTINGS::sendCommand(MV_SETSKYBOX, 0);
	      #endif   
	      skybox = sky; 
   }
	static inline MVNODE *getSkybox()                        { return skybox; }
   static inline void wireframeOn()                         
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_WIREFRAMEON, 0);
         #endif	   
         wireframe = true; 
   }
   static inline void wireframeOff()                        
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_WIREFRAMEOFF, 0);
         #endif	      
         wireframe = false; 
   }
   static inline bool isWireframed()                        { return wireframe; }
   static inline void shadowingOn()                         
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_SHADOWINGON, 0);
         #endif	
         if (MVSHADER::useShadows())
               shadowing = true; 
   }
   static inline void shadowingOff()                        
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_SHADOWINGOFF, 0);
         #endif	
         shadowing = false; 
   }     
   static inline bool isShadowed()                          { return shadowing; }
   static inline void bloomingOn()                          
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_BLOOMINGON, 0);
         #endif	               
         blooming = true; 
   }
   static inline void bloomingOff()                         
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_BLOOMINGOFF, 0);
         #endif	               
         blooming = false; 
   }
   static inline bool isBlooming()                          { return blooming; }
   static inline void setBloomingIntensity(float i)         
   { 
         #ifdef MV_CAVE
            int _i; memcpy(&_i, &i, sizeof(int));
	         MVSETTINGS::sendCommand(MV_BLOOMINGINTENSITY, _i);
         #endif	         
         bloomingIntensity = i; 
   }   
   static inline float getBloomingIntensity()               { return bloomingIntensity; }   
   static inline void HDROn()
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_HDRON, 0);
         #endif	 
			HDR = true;
   }
   static inline void HDROff()
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_HDROFF, 0);
         #endif	 
			HDR = false;
   }
   static inline bool isHDR()											{ return HDR; }
   static inline void setHDRExposure(float e)					
   {
			HDRExposure = e;
   }
   static inline float getHDRExposure()							{ return HDRExposure; }  
   static inline void DOFOn()
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_DOFON, 0);
         #endif	   
			DOF = true;			
   }
   static inline void DOFOff()
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_DOFOFF, 0);
         #endif	   
			DOF = false;
   }
   static inline void setDOFFocus(float f)
   {
         #ifdef MV_CAVE
            int _f; memcpy(&_f, &f, sizeof(int));
	         MVSETTINGS::sendCommand(MV_DOFFOCUS, _f);
         #endif	   
			DOFFocus = f;			
   }
   static inline void setDOFRange(float r)
   {
         #ifdef MV_CAVE
            int _r; memcpy(&_r, &r, sizeof(int));
	         MVSETTINGS::sendCommand(MV_DOFRANGE, _r);
         #endif	   
			DOFRange = r;			
   }
   static inline float getDOFFocus()  								{ return DOFFocus; }
   static inline float getDOFRange()  								{ return DOFRange; }
   static void updateTimer();
   static inline void updateVideo()                        
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_UPDATEVIDEO, 0);
         #endif	               
   }
   static inline void skipShadowsGeneration()                        
   { 
         skipShadows = true;
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_SKIPSHADOWS, 0);
         #endif	               
   }   
	static bool clear(bool color, bool zeta, bool stencil = false);	
	static bool begin2D();
	static bool end2D();
	static bool begin3D(MVCAMERA *camera);
	static bool end3D(bool swapFBO = true);			
	static MVID select(int x, int y);	
	static bool swap();
	
//////////////
	private: //
//////////////	

	// Status:
	static bool active;	

	// Management:	
	static int timer;
	static long timeStamp;	
	static unsigned videoMemoryTotal;
	static unsigned videoMemoryUsed;
	
   // Flags:
   static bool shadowing;
   static bool wireframe;
   static bool displayNormals;
   static bool blooming;
   static float bloomingIntensity;  
   static bool HDR;
   static float HDRExposure;   
   static float HDRCurrentLuminance;
   static bool DOF;
   static float DOFFocus;
   static float DOFRange;
   static bool skipShadows;
   static bool renderingOnFbo;

	// Path:
	static char mvePath[MV_MAXNUMBEROFCHARS];
	
	// Skybox:
	static MVNODE *skybox;		
} MVISIO; // MVisio main class



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
