//////////////////////////////////////////////
//                                          //
// project  : mVISIO v1.6a                  //
// file     : mvisio_anim.H					  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005-2006            //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

#define MV_MAXNUMBEROFBONESANIM	128
#define MV_MAXNUMBEROFFRAMESANIM 1024
	
typedef struct MVBONEANIM
{
   char name[MV_MAXNUMBEROFCHARS];   
} MVBONEANIM;

typedef struct MVBONEFRAME
{
   MVVECTOR3 rotation;
   MVVECTOR3 position;
   char key;
} MVBONEFRAME;



//////////////////
// CLASS MVANIM //
//////////////////

///////////////////////////////////////////
typedef class MV_API MVANIM // Animation //
///////////////////////////////////////////
{   	
/////////////
	public: //	
/////////////

	// Cons/dest:
	MVANIM();
	~MVANIM();			
	
	// Management:	
	static bool cleanUp();	
	static unsigned getNumberOfAnims();	
	static MVANIM *findByFilename(char *name);
	static MVANIM *findById(MVID id);
	
	// Settings:
	inline void setNumberOfFrames(unsigned int f)            { numberOfFrames = f; }
	inline void setNumberOfBones(unsigned int b)             { numberOfBones = b; }
	inline void setTranslating(bool t)                       { useTranslation = t; }
	inline bool getTranslating() const                       { return useTranslation; }	
	inline MVWORD getNumberOfFrames() const                  { return numberOfFrames; }
	inline MVWORD getNumberOfBones() const                   { return numberOfBones; }
	inline bool setBoneName(unsigned int b, char *name)		
	{
			if (b < MV_MAXNUMBEROFBONESANIM && name != NULL)
			{
					strcpy(skeleton[b].name, name);
			}
			return false;
	}
	inline char *getBoneName(unsigned int b)                 { return skeleton[b].name; }
	inline MVBONEFRAME *getKeyframe(unsigned frame, unsigned bone)   { return &keyframe[frame][bone]; }	
	
	// Loading:
	static void setMVAPath(char *path)								{ if (path) strcpy(mvaPath, path); else strcpy(mvaPath, ""); }
	static char *getMVAPath() 											{ return mvaPath; }
	bool load(char *filename);
	bool save(char *filename);
	
	// Interpolation:
	void reset();
	int findBoneByName(char *name);
	bool acquireStructure(MVMESH *mesh);
	bool setFrame(unsigned frame, unsigned bone, float rotX, float rotY, float rotZ);
	bool setFrameRotationX(unsigned frame, unsigned bone, float rotX);
	bool setFrameRotationY(unsigned frame, unsigned bone, float rotY);
	bool setFrameRotationZ(unsigned frame, unsigned bone, float rotZ);
	MVVECTOR3 getFrameRotation(unsigned frame, unsigned bone)
	{
			return keyframe[frame][bone].rotation;
	}
	bool unsetFrame(unsigned frame, unsigned bone);
	bool createKeyframes(unsigned int first, unsigned int last);
	
	// Rendering:
	bool apply(unsigned int frame, MVNODE *root);
	
//////////////
	private: //
//////////////

	// Data: 
	MVID id;
	char filename[MV_MAXNUMBEROFCHARS];	
	bool useTranslation;
	MVWORD numberOfBones, numberOfFrames;
	MVBONEANIM skeleton[MV_MAXNUMBEROFBONESANIM];
	MVBONEFRAME keyframe[MV_MAXNUMBEROFFRAMESANIM][MV_MAXNUMBEROFBONESANIM];
	
	// List:
	static MVANIM *firstAnim;
	MVANIM *prev, *next;  	   
	
	// Path:
	static char mvaPath[MV_MAXNUMBEROFCHARS];
} MVANIM; // Anim
