//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_box.H						  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Default:
	#define MV_BOX_DEFAULT_COLOR	       0.5f, 0.5f, 0.5f, 1.0f
	#define MV_BOX_DEFAULT_THICKNESS     3



//////////////////////
// CLASS MVBOXCOORD //
//////////////////////

/////////////////////////////////////////////////////
typedef class MV_API MVBOXCOORD // Box coordinates //
/////////////////////////////////////////////////////
{
/////////////
   public: //
/////////////

   union
   {
  	   float d[8]; 
	   struct
		{
			float u1, v1;
			float u2, v2;
			float u3, v3;
			float u4, v4;			
		};
	};
	
	MVBOXCOORD() 
	{ 
	   u1 = 0.0f; v1 = 1.0f;
	   u2 = 1.0f; v2 = 1.0f;
	   u3 = 0.0f; v3 = 0.0f;
	   u4 = 1.0f; v4 = 0.0f;
	}
	MVBOXCOORD(float u, float v, bool invert = false)
	{
	   if (invert == false)
	   {	   
	         u1 = 0.0f; v1 = v;
	         u2 = u; v2 = v;
	         u3 = 0.0f; v3 = 0.0f;
	         u4 = u; v4 = 0.0f;
	   }
	   else
	   {
	         u1 = 0.0f; v1 = 0.0f;
	         u2 = u; v2 = 0.0f;
	         u3 = 0.0f; v3 = v;
	         u4 = u; v4 = v;	         
	   }
	}	
} MVBOXCOORD;



/////////////////
// CLASS MVBOX //
/////////////////

///////////////////////////////////////////////////////
typedef class MV_API MVBOX : public MVNODE // Box 2D //
///////////////////////////////////////////////////////
{
/////////////
	public: //
/////////////

	// Const/Dest:
	MVBOX();
	virtual ~MVBOX();
	
	// Settings:
	inline void setSizeX(int x)												{ sizeX = x; }
	inline void setSizeY(int y)												{ sizeY = y; }	
	inline void setSizes(int x, int y)										{ sizeX=x; sizeY=y; }
	inline void setColor(float r, float g, float b, float a)			{ color.set(r, g, b, a); }
	inline void setColor(MVCOLOR4 c)  										{ color = c; }	
	inline void setThickness(unsigned t)									{ thickness = t; }	
	inline void setCheckStatus(bool c)										{ checkStatus = c; }
	inline void setDisplayStatus(bool d)									{ displayStatus = d; }
	inline void setStaticStatus(bool s)										{ staticStatus = s; }
	inline void setClickedStatus(bool c)									{ clickedStatus = c; }	
	inline void setLimitStatus(bool l)										{ limitStatus = l; }
	inline void setScissorStatus(bool s)                           { scissorStatus = s; }
	inline void setBoxCoords(MVBOXCOORD &b)                        { boxCoord = b; }
	inline bool isTransparent()												{ return color.a != 1.0f; }	
	inline bool isStatic()														{ return staticStatus; }	
	inline bool isClicked()														{ return clickedStatus; }
	bool setTexture(MVTEXTURE *tex);	
   bool setShader(MVSHADER *shad);
	inline MVVECTOR2 getSizes()                                    { return MVVECTOR2((float) sizeX, (float)sizeY); }
	inline int getSizeX() const   											{ return sizeX; }
	inline int getSizeY() const												{ return sizeY; }		
	inline MVCOLOR4 getColor() const											{ return color; }	
	inline unsigned getThickness() const									{ return thickness; }
	inline bool getCheckStatus() const										{ return checkStatus; }
	inline bool getDisplayStatus() const									{ return displayStatus; }	
	inline bool getClickedStatus() const									{ return clickedStatus; }
	inline bool getLimitStatus() const										{ return limitStatus; }
	inline bool getScissorStatus() const                           { return scissorStatus; }
	inline long getForemostTimeStamp() const								{ return foremostTimeStamp; }
	inline int getStartX() const												{ return startX; }
	inline int getStartY() const												{ return startY; }
	inline int getEndX() const													{ return endX; }
	inline int getEndY() const													{ return endY; }
	inline MVBOXCOORD getBoxCoord()                                { return boxCoord; }
	static inline long getNextCounter()										{ return ++foremostCounter; }
	inline void setLimits(int sx, int sy, int ex, int ey)				{ startX=sx; startY=sy; endX=ex; endY=ey; }
	
	// Management:
	bool checkBoundaries(int x, int y);
	bool checkLimits(int *x, int *y);
	bool setForemost(long newForemostTimeStamp);
	
	// Rendering:
	bool render(void *data = NULL);

////////////////
	protected: //
////////////////	

	// Static:
	static long foremostCounter;

	// Data:
	int sizeX, sizeY;	
	int startX, startY, endX, endY;
	unsigned thickness;
	MVCOLOR4 color;	
	bool checkStatus, displayStatus, staticStatus, clickedStatus, limitStatus, scissorStatus;
	long foremostTimeStamp;
	MVTEXTURE *texture;
   MVSHADER *shader;
	MVBOXCOORD boxCoord;
} MVBOX; // Box 2D
