//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_element.H				  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////




//////////////
// #DEFINES //
//////////////

   // Limits:
   #ifndef MV_PDA
      #define MV_MAXNUMBEROFELEMENTS 4096   
   #else
      #define MV_MAXNUMBEROFELEMENTS 1024
   #endif            

   // Defaults:
   #define MV_ELEMENT_DEFAULT_CLIPPINGRADIUS 128.0f    



/////////////////////
// CLASS MVELEMENT //
/////////////////////

//////////////////////////////////////////////////////
typedef class MV_API MVELEMENT // Rendering element //
//////////////////////////////////////////////////////
{ 
	friend class MVISIO; 
	friend class MVLIGHT;
	friend class MVTEXTURE;

/////////////
	public: //	
/////////////

	// Cons/dest:
	MVELEMENT();
	~MVELEMENT();		
	
	// Init/free:
	static inline bool isActive()									{ return active; }
	static bool init(unsigned elements);
	static bool free();		
	
	// Settings:
   static inline void clippingOn()                       { clipping = true; }
   static inline void clippingOff()                      { clipping = false; }
   static inline void setClippingRadius(float r)         { clippingRadius = r; }   
   inline void setParam1(unsigned int p)                 { param1 = p; }
   static inline float getClippingRadius()               { return clippingRadius; }
	static inline unsigned getNumberOfElements()				{ return numberOfElements; }
	static inline unsigned getNumberOfPassedElements()    { return numberOfPassedElements; }
	static inline unsigned getNumberOfLights()				{ return numberOfLights; }
	inline MVNODE *getNode()										{ return node; }
	inline float getDistance()										{ return distance; }
	inline MVID getAuxId()											{ return auxId; }
	inline MVMATRIX *getMatrix()									{ return &matrix; }
	static MVELEMENT *getLight(unsigned c)					   { if (numberOfLights && c<numberOfLights) return elementPointer[c]; else return NULL; }	
   inline void setNearestLight(unsigned c, MVELEMENT *l) { nearestLight[c] = l; }
	inline MVELEMENT *getNearestLight(unsigned c)			{ return nearestLight[c]; }	
	inline unsigned int getParam1()								{ return param1; }		
	
	// Management:
	static void reset();
	static bool pass(MVNODE *node, bool recurse = true);
	static bool passSkeleton(MVNODE *node, MVMESH *rootMesh);
	static bool sort2D();
	static bool sort3D();
	static bool sortLights(MVVECTOR3 position);		
	static bool manageGUIEvents(char key, int mouseX, int mouseY, int mouseBL, int mouseBR, MVID *id);
	
//////////////
	private: //
//////////////	

	// Stats:
	static bool active; 
	static unsigned numberOfElements;	
	static unsigned numberOfPassedElements;
	static unsigned numberOfLights;	
	static unsigned numberOfTransparencies;

	// Common data:
	static MVELEMENT *element;
	static MVELEMENT **elementPointer;	      

   // Clipping:
   static bool clipping;
   static float clippingRadius;
	
	// Data:
	MVNODE *node;
	MVMATRIX matrix;
	MVID auxId;
	float distance;
	MVELEMENT *nearestLight[MV_MAXNUMBEROFLIGHTS];
	unsigned int param1;	
} MVELEMENT; // Rendering element
