///////////////////////////////////////////////
//                                           //
// project  : mVISIO v1.6a                   //
// file     : mvisio_emitter.H               //
// authors  : Achille Peternier, Loc Duboux //
// copyright: EPFL (C) 2005-2007             //
//                                           //
///////////////////////////////////////////////




/////////////
// #DEFINE //
/////////////

   // Limits:
   #ifdef MV_PDA
      #define MV_MAXNUMBEROFPARTICLES 1024
      #define MV_MAXNUMBEROFPARTICLESPEREMITTER 512
   #else
      #define MV_MAXNUMBEROFPARTICLES 8192
      #define MV_MAXNUMBEROFPARTICLESPEREMITTER 4096
   #endif

   // Kinds:
   typedef enum MV_EMITTER_TYPE
   {
      MV_EMITTER_SPHERICAL = 0,
      MV_EMITTER_PLANAR,
      MV_EMITTER_CIRCULAR,
   } MV_EMITTER_TYPE;



//////////////////////
// CLASS MVPARTICLE //
//////////////////////

////////////////////////////////////////////////
typedef class MVPARTICLE // A simple particle //
////////////////////////////////////////////////
{
/////////////
   public: //
/////////////

   MVPARTICLE() { used = false; };

   // Data:
   MVVECTOR3 position, oldPosition, velocity, custom;
   MVCOLOR4 color;
   float spin, spinning, weight, size, startEnergy, energy;
   bool used;
} MVPARTICLE;



/////////////////////
// CLASS MVEMITTER //
/////////////////////

////////////////////////////////////////////////////////////////////
typedef class MV_API MVEMITTER : public MVNODE // Particle system //
////////////////////////////////////////////////////////////////////
{
   friend class MVELEMENT;

/////////////
   public: //
/////////////

   // Cons/dest:
   MVEMITTER();
   ~MVEMITTER();
   static bool init();
   static bool free();

   // Settings:
   inline void setNumberOfParticles(int particles);
   inline unsigned getNumberOfParticles() const                       { return numberOfParticles; }
   inline void setTexture(MVTEXTURE *tex)
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETTEXTURE, id, tex->getId());
         #endif
         texture = tex;
   }
   inline MVTEXTURE *getTexture() const                               { return texture; }
   inline void setEmitterType(int type)                               
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETEMITTERTYPE, id, type);   
         #endif 	  
         typeOfEmitter = type;
   }
   inline int  getEmitterType()                                       { return typeOfEmitter;}
   inline void setMaximumEnergy(unsigned maxEnergy)                   
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETMAXIMUMENERGY, id, maxEnergy);   
         #endif    
         this->maxEnergy = (maxEnergy>1) ? maxEnergy : 1;
   }
   inline unsigned getMaximumEnergy()                                 { return maxEnergy;}
   inline void setMaximumVelocity(unsigned maxVelocity)               
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETMAXIMUMVELOCITY, id, maxVelocity);   
         #endif
         this->maxVelocity = (maxVelocity>1)? maxVelocity: 1;
   }
   inline unsigned getMaximumVelocity()                               { return maxVelocity;}
   inline void setMaximumParticuleSize(unsigned size)                
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETMAXIMUMPARTICLESIZE, id, size);   
         #endif
         this->maxPartSize = (size>1) ? size : 1;
   }
   inline unsigned getMaximumParticuleSize()                          { return maxPartSize;}
   inline void setMaximumSpin(unsigned spin)                          
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETMAXIMUMSPIN, id, spin);   
         #endif
         maxSpin = (spin>1)? spin: 1;
   }
   inline unsigned getMaximumSpin()                                   { return maxSpin;}
   inline void setMaximumAngle(float angle)                           { maxAngle = angle;}
   inline float getMaximumAngle()                                     { return maxAngle;}
   inline void setSpinning(float spinning)                            
   { 
         #ifdef MV_CAVE
	         int _i; memcpy(&_i, &spinning, sizeof(float));
            MVSETTINGS::sendValue(MV_EMITTER_SETSPINNING, id, _i);   
         #endif
         this->spinning = spinning;
   }
   inline float getSpinning()                                         { return spinning;}
   inline void setGravity(MVVECTOR3 &gravity)                         
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_EMITTER_SETGRAVITY, id, gravity);   
         #endif 
         this->gravity = gravity;
   }
   inline MVVECTOR3 getGravity()                                      { return gravity;}
   void setColorStart(MVVECTOR3 &minColor);
   inline MVVECTOR3 getColorStart()                                   { return colorStart;}
   void setColorEnd(MVVECTOR3 &maxColor);
   inline MVVECTOR3 getColorEnd()                                     { return colorEnd;}
   inline void setCenter(MVVECTOR3 &center)                           
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_EMITTER_SETCENTER, id, center);   
         #endif 	            
         this->center = center;
   }
   inline MVVECTOR3 getCenter()                                       { return center;}
   inline void setUpVector(MVVECTOR3 &upVec)                          
   {
         #ifdef MV_CAVE
	         MVSETTINGS::sendVector3(MV_EMITTER_SETUPVECTOR, id, upVec);   
         #endif 	   
         this->upVec = upVec;
   }
   inline MVVECTOR3 getUpVector()                                     { return upVec;}
   inline void setSizeX(float sizeX)                                  
   { 
         #ifdef MV_CAVE
	         int _i; memcpy(&_i, &sizeX, sizeof(float));
            MVSETTINGS::sendValue(MV_EMITTER_SETSIZEX, id, _i);   
         #endif	
         this->emitterSizeX = sizeX;
   }
   inline float getSizeX()                                            { return emitterSizeX;}
   inline void setSizeY(float sizeY)                                  
   { 
         #ifdef MV_CAVE
	         int _i; memcpy(&_i, &sizeY, sizeof(float));
            MVSETTINGS::sendValue(MV_EMITTER_SETSIZEY, id, _i);   
         #endif	
         this->emitterSizeY = sizeY;
   }
   inline float getSizeY()                                            { return emitterSizeY;}
   inline void setRadius(float radius)                                
   { 
         #ifdef MV_CAVE
	         int _i; memcpy(&_i, &radius, sizeof(float));
            MVSETTINGS::sendValue(MV_EMITTER_SETRADIUS, id, _i);   
         #endif
         this->radius = radius;
   }
   inline float getRadius()                                           { return radius;}
   inline void setBlendingSrc(GLenum mode)                            
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETBLENDINGSRC, id, mode);   
         #endif	 
         this->blendingSrc = mode;
   }
   inline GLenum getBlendingSrc()                                     { return blendingSrc;}
   inline void setBlendingDst(GLenum mode)                            
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_EMITTER_SETBLENDINGDST, id, mode);   
         #endif	
         this->blendingDst = mode;
   }
   inline GLenum getBlendingDst()                                     { return blendingDst;}

   // Management:
   bool load(char *filename);

   // Rendering:
   bool initialize();
   bool render(void *data);

//////////////
   private: //
//////////////

   // Data:
   int typeOfEmitter;        
   bool active;              
   unsigned numberOfParticles;
   int index[MV_MAXNUMBEROFPARTICLESPEREMITTER];     

   // Settings:
   unsigned maxEnergy;                               
   unsigned maxVelocity;                             
   unsigned maxPartSize;                             
   unsigned maxSpin;                                 
   float spinning;                                   

   // Physics:
   MVVECTOR3 center;          
   MVVECTOR3 gravity;         

   // Config:
   float maxAngle;                                     
   float emitterSizeX, emitterSizeY;                   
   float radius;                                       
   MVVECTOR3 upVec;                                    

   // Colors:
   MVVECTOR3 colorSpace;                               
   MVVECTOR3 colorStart, colorEnd;                     
   MVTEXTURE *texture;                                 

   // Blending mode:
   GLenum blendingSrc;                                 
   GLenum blendingDst;                                 

   // Shared:
   static MVPARTICLE *particle;                                           
   static MVVECTOR4 vertexArray[MV_MAXNUMBEROFPARTICLESPEREMITTER*4];       
   static unsigned char colorArray[MV_MAXNUMBEROFPARTICLESPEREMITTER*4*4];  
   static MVUV uvArray[MV_MAXNUMBEROFPARTICLESPEREMITTER*4];                
   static MVUV uvSecondaryArray[MV_MAXNUMBEROFPARTICLESPEREMITTER*4];       
   static int numberbOfParticlesUsed;

   // For shader:
   MVVECTOR3 normalArray[MV_MAXNUMBEROFPARTICLESPEREMITTER*4];  

   // Methods:
   bool updateParticle(MVPARTICLE *part);                       
} MVEMITTER; // Particle system