//////////////////////////////////////////////
//                                          //
// project  : mVISIO v1.6.6a                //
// file     : mvisio_fbo.H						  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2009                 //
//												        //
//////////////////////////////////////////////



///////////
// ENUMS //
///////////

	// Operations:
	typedef enum MV_FBO_BINDOPERATIONS
	{	
		MV_FBO_BINDCOLORTEXTURE,
		MV_FBO_BINDDEPTHTEXTURE,	
		MV_FBO_BINDDEPTHBUFFER,	
	} MV_FBO_BINDOPERATIONS;	



/////////////////
// CLASS MVFBO //
/////////////////

/////////////////////////////////////
typedef class MV_API MVFBO  // FBO //
/////////////////////////////////////
{
/////////////
	public: //
/////////////

	// Const/Dest:
	MVFBO();
	~MVFBO();		
	
	// Settings:
	inline MVID getId()														{ return id; }		
	inline char *getName()													{ return name; }	
	inline GLuint getGLId()													{ return glId; }			
	inline MVTEXTURE *getTexture()									   { return texture; }
	inline int getSizeX()													{ return sizeX; }
	inline int getSizeY()													{ return sizeY; }
	
	// Management:
	static bool cleanUp();	
	static unsigned getNumberOfFbos();
	bool isOk();	
	
	// FBO related and FX:	
	bool blur(MVFBO *output, MVSHADER *combiner, int level = 2);	
	bool computeLuminance(float *luminance); 
	
	// Rendering:
	bool bindTexture(int operation, MVTEXTURE *texture);
	bool bindRenderBuffer(int operation, int sizeX, int sizeY);
	bool render(void *data = NULL);
	static void unbind();
	static inline MVFBO *getCurrent()									{ return currentFbo; }
		
	
//////////////
	private: //
//////////////	
	
	// Data:
	MVID id;									// FBO id
	char name[MV_MAXNUMBEROFCHARS];	// FBO name
	MVFBO *prev, *next;					// Linked list chain
	MVTEXTURE *texture;					// Optional texture for render2texture
	int sizeX, sizeY;						// FBO sizes
	GLuint glId;							// OGL framebuffer id
	GLuint glIdRenderBuffer;			// OGL optional render buffer id

	// Data:
	static MVFBO *currentFbo;			// FBO cache
	static MVFBO *firstFbo;				// Linked list of FBOs
	static int viewport[4];				// Previous viewport for restoring
} MVFBO; // FBO
