//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_font.H					  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Limits:
	#define MV_FIRSTCHARPERFONT 32
	#define MV_LASTCHARPERFONT  127
	#define MV_MAXNUMBEROFCHARSPERFONT MV_LASTCHARPERFONT-MV_FIRSTCHARPERFONT	
	#define MV_FONTCACHESIZE 1024
	
	// Default:
	#define MV_FONT_DEFAULT_CHARSIZE 16

		
	
//////////////////
// CLASS MVFONT //
//////////////////

/////////////////////////////////////////
typedef class MV_API MVFONT // Font 2D //
/////////////////////////////////////////
{
/////////////
	public: //
/////////////

	// Const/Dest:
	MVFONT();
	~MVFONT();	
	static bool init();
	static bool free();
	
	// Settings:
	inline void setName(char *n)											{ if (n && MV_VALIDSTRING(n)) strcpy(name, n); }
	inline void setCharSizeX(unsigned x)								{ charSizeX = x; }
	inline void setCharSizeY(unsigned y)								{ charSizeY = y; }
	bool setTexture(MVTEXTURE *tex);	
	inline MVID getId()														{ return id; }
	inline char *getName()													{ return name; }	
	inline unsigned getCharSizeX()										{ return charSizeX; }
	inline unsigned getCharSizeY()										{ return charSizeY; }
	
	// Kerning:
	bool setKerning(char c, char offset);
	char getKerning(char c);
	bool loadKerning(char *filename);
	unsigned stringLength(char *text);
	int format(char *inputText, char *outputText, unsigned sizeX, unsigned sizeY);
	
	// Loading:
	static void setPath(char *path)										{ if (path) strcpy(fontPath, path); else strcpy(fontPath, ""); }
	static char *getPath() 													{ return fontPath; }
	
	// Management:
	static bool cleanUp();
	static inline void reset()                                  { fontCachedCurrent = 0; }
	static int getNumberOfFonts();
	static MVFONT *findByName(char *name);
	static MVFONT *findById(MVID id);
	static MVFONT *getFirst()                                   { return firstFont; }
	MVFONT *getNext()                                           { return next; }
	MVFONT *getPrevious()                                       { return prev; }
	
	// Rendering:	
	bool putChar(int x, int y, char c, bool bindStatus = true);	
	bool putString(int x, int y, char *text, ...);

//////////////
	private: //
//////////////

	// Static:
	static MVFONT *firstFont;	
	
	// Path:
	static char fontPath[MV_MAXNUMBEROFCHARS];

	// Data:
	MVID id;
	MVTEXTURE *texture;
	char kerning[MV_MAXNUMBEROFCHARSPERFONT];
	unsigned charSizeX, charSizeY;

	// Data:
	char name[MV_MAXNUMBEROFCHARS];
	
	// Lists:
	MVFONT *prev, *next;		
	
	// Font cached character:
	static NATIVE_TYPE_CAST *fontCachedVertex;
	static NATIVE_TYPE_CAST *fontCachedTexCoord;
	static unsigned char *fontCachedColor;
	static unsigned short *fontCachedIndex;
	static unsigned int fontCachedCurrent;
} MVFONT; // Font 2D
