//////////////////////////////////////////////
//                                          //
// project  : mVISIO v1.6a                  //
// file     : mvisio_light.H					  //
// author   : Achille Peternier             // 
// copyright: VRLab/EPFL (C) 2005-2007      //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Limits:
	#define MV_MAXNUMBEROFLIGHTS           4

	// Default:
	#define MV_LIGHT_DEFAULT_AMBIENT	      0.2f, 0.2f, 0.2f, 1.0f
	#define MV_LIGHT_DEFAULT_DIFFUSE	      0.7f, 0.7f, 0.7f, 1.0f
	#define MV_LIGHT_DEFAULT_SPECULAR      0.2f, 0.2f, 0.2f, 1.0f
	#define MV_LIGHT_DEFAULT_CUTOFF	      180.0f
	#define MV_LIGHT_DEFAULT_EXPONENT      0.0f
	#define MV_LIGHT_DEFAULT_DIRECTION     0.0f, 0.0f, 0.0f
	#define MV_LIGHT_DEFAULT_SHADOWMAPSIZE 1024
	#define MV_LIGHT_DEFAULT_KATTENUATION  1.0f
	#define MV_LIGHT_DEFAULT_LATTENUATION  0.0f
	#define MV_LIGHT_DEFAULT_QATTENUATION  0.0f
	#define MV_LIGHT_DEFAULT_SHADOWNEARPLANE MV_OPENGL_DEFAULT_NEARPLANE
	#define MV_LIGHT_DEFAULT_SHADOWFARPLANE  MV_OPENGL_DEFAULT_FARPLANE
	
	// Light type:
	typedef enum MV_LIGHT_TYPE
	{
	   MV_LIGHT_OMNI = 0,
	   MV_LIGHT_DIRECTIONAL,
	   MV_LIGHT_SPOT,
	} MV_LIGHT_TYPE;
	
	

///////////////////
// CLASS MVLIGHT //
///////////////////

////////////////////////////////////////////////////////
typedef class MV_API MVLIGHT : public MVNODE // Light //
////////////////////////////////////////////////////////
{  	
	friend class MVMESH; // REMOVE THAT ONE DAY
	
/////////////
	public: //	
/////////////

	// Cons/dest:
	MVLIGHT();
	~MVLIGHT();	
	
	// Settings:
	inline void setLightType(MV_LIGHT_TYPE l)                   
	{ 
			#ifdef MV_CAVE
				MVSETTINGS::sendValue(MV_LIGHT_SETLIGHTTYPE, id, l);
			#endif
			lightType = l; 
	}
	inline void setAmbient(MVCOLOR4 col)    							
	{ 	      
	      #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_LIGHT_SETAMBIENT, id, col.r, col.g, col.b, col.a);
         #endif	
	      ambient = col; 
	}
   inline void setDiffuse(MVCOLOR4 col)								
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_LIGHT_SETDIFFUSE, id, col.r, col.g, col.b, col.a);
         #endif	
         diffuse = col; 
   }
   inline void setSpecular(MVCOLOR4 col)								
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_LIGHT_SETSPECULAR, id, col.r, col.g, col.b, col.a);
         #endif	
         specular = col; 
   }
   inline void setAmbient(float r, float g, float b, float a)  
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_LIGHT_SETAMBIENT, id, r, g, b, a);
         #endif	
         ambient.set(r, g, b, a); 
   }
   inline void setDiffuse(float r, float g, float b, float a)  
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_LIGHT_SETDIFFUSE, id, r, g, b, a);
         #endif	
         diffuse.set(r, g, b, a); 
   }
   inline void setSpecular(float r, float g, float b, float a) 
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_LIGHT_SETSPECULAR, id, r, g, b, a);
         #endif	
         specular.set(r, g, b, a); 
   }
	inline void setCutOff(float c)										
	{ 
	      #ifdef MV_CAVE
	            int _c; memcpy(&_c, &c, sizeof(int));
	            MVSETTINGS::sendValue(MV_LIGHT_SETCUTOFF, id, _c);	            
         #endif	            
	      cutOff = c; 
	}		
	inline void setExponent(float e)                            
	{ 
	      #ifdef MV_CAVE
	            int _e; memcpy(&_e, &e, sizeof(int));
	            MVSETTINGS::sendValue(MV_LIGHT_SETEXPONENT, id, _e);	            
	      #endif
	      exponent = e; 
	}
	inline void setExclusive(bool b)										{ exclusive = b; }
	inline void setAttenuation(float k, float l, float q)       { attenuation.set(k, l, q); }
	inline void setShadowNearPlane(float n)                     { shadowNearPlane = n; }
	inline void setShadowFarPlane(float f)                      { shadowFarPlane = f; }
	inline MV_LIGHT_TYPE getLightType()                         { return lightType; }
	inline MVCOLOR4 getAmbient()					      				{ return ambient; }
	inline MVCOLOR4 getDiffuse()										   { return diffuse; }
	inline MVCOLOR4 getSpecular()										   { return specular; }
	inline MVVECTOR3 getAttenuation()                           { return attenuation; }	
	inline float getCutOff()												{ return cutOff; }
	inline float getExponent()                                  { return exponent; }
	inline float getShadowNearPlane()                           { return shadowNearPlane; }
	inline float getShadowFarPlane()                            { return shadowFarPlane; }
	inline bool isExclusive()												{ return exclusive; }
	static inline void lightingOff()										{ if (lighting) { glDisable(GL_LIGHTING); lighting = false; } }
	static inline void lightingOn()										{ if (!lighting) { glEnable(GL_LIGHTING); lighting = true; } }		
	static inline bool getLighting()                            { return lighting; }
	
	// Loading:
	static MVNODE *load(char *buffer, long *position);   
	
	// Shadows:
	inline void castShadows(bool s)                             
	{ 
	      #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_LIGHT_CASTSHADOWS, id, (int) s);
         #endif	
	      shadowCaster = s; 
	}
	inline bool isShadowCaster()                                { return shadowCaster; }
	
	// Rendering:
	static bool activateNearest(MVELEMENT *elem);
	bool render(void *data);
	
//////////////
//	private: //
//////////////				

	// Specifics:	
	MV_LIGHT_TYPE lightType;
	MVCOLOR4 ambient, diffuse, specular;	
	MVVECTOR3 attenuation;
	float cutOff, exponent;	
	bool exclusive, shadowCaster;
	float shadowNearPlane, shadowFarPlane;
	
	// Shadow maps:
	MVTEXTURE *shadowMap;
	
	// Shared:
	static bool lighting;
} MVLIGHT; // Light

