//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_material.H				  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Default:
	#define MV_MATERIAL_DEFAULT_AMBIENT	       0.2f, 0.2f, 0.2f, 1.0f
	#define MV_MATERIAL_DEFAULT_DIFFUSE	       0.6f, 0.6f, 0.6f, 1.0f
	#define MV_MATERIAL_DEFAULT_SPECULAR	    0.2f, 0.2f, 0.2f, 1.0f
	#define MV_MATERIAL_DEFAULT_EMISSION	    0.0f, 0.0f, 0.0f, 1.0f
	#define MV_MATERIAL_DEFAULT_TRANSPARENCY   1.0f
	#define MV_MATERIAL_DEFAULT_SHININESS      0.0f
	
	

//////////////////////
// CLASS MVMATERIAL //
//////////////////////

//////////////////////////////////////////////
typedef class MV_API MVMATERIAL // Material //
//////////////////////////////////////////////
{   
/////////////
	public: //	
/////////////

	// Cons/dest:
	MVMATERIAL();
	~MVMATERIAL();	
	
	// Copy operator (remove it?):
	inline void clone(MVMATERIAL *mat)
	{		
			if (mat == NULL) return;
			sprintf(name, "%s%d", mat->name, id);			
			ambient = mat->ambient;
			diffuse = mat->diffuse;
			specular = mat->specular;
			emission = mat->emission;
			transparency = mat->transparency;
			shininess = mat->shininess;
			memcpy(texture, mat->texture, sizeof(MVTEXTURE *)*MV_TEXTURE_MAXNUMBEROFCHANNELS);
			shader = mat->shader;
	}			
	
	// Settings:
	inline void incReferences()											{ references++; }
	inline void decReferences()											{ references--; }
	inline int  getReferences()											{ return references; }
	inline MVID getId()	const												{ return id; }
	inline void setName(char *n)											{ if (n && MV_VALIDSTRING(n)) strcpy(name, n); }
	inline char *getName()													{ return name; }		
	inline int getTimeStamp() const										{ return timeStamp; }		
	inline void setAmbient(MVCOLOR4 &col)								   
	{ 
	      #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETAMBIENT, id, col.r, col.g, col.b, col.a);
         #endif	
	      ambient = col; 
	}
   inline void setDiffuse(MVCOLOR4 &col)								   
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETDIFFUSE, id, col.r, col.g, col.b, col.a);
         #endif	
         diffuse = col; 
   }
   inline void setSpecular(MVCOLOR4 &col)								
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETDIFFUSE, id, col.r, col.g, col.b, col.a);
         #endif	
         specular = col; 
   }
   inline void setEmission(MVCOLOR4 &col)								
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETEMISSION, id, col.r, col.g, col.b, col.a);
         #endif	
         emission = col; 
   }
   inline void setAmbient(float r, float g, float b, float a)  
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETAMBIENT, id, r, g, b, a);
         #endif	
         ambient.set(r, g, b, a); 
   }
   inline void setDiffuse(float r, float g, float b, float a)  
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETDIFFUSE, id, r, g, b, a);
         #endif	
         diffuse.set(r, g, b, a); 
   }
   inline void setSpecular(float r, float g, float b, float a) 
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETSPECULAR, id, r, g, b, a);
         #endif	
         specular.set(r, g, b, a); 
   }         
   inline void setEmission(float r, float g, float b, float a) 
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector4(MV_MATERIAL_SETEMISSION, id, r, g, b, a);
         #endif	
         emission.set(r, g, b, a); 
   }
	inline void setTransparency(float t)								{ transparency = t; }
	inline void setShininess(float s)							      
	{ 
	      #ifdef MV_CAVE
	            int _s; memcpy(&_s, &s, sizeof(int));
	            MVSETTINGS::sendValue(MV_MATERIAL_SETSHININESS, id, _s);	            
         #endif	 
	      shininess = s; 
	}				
	bool setTexture(unsigned c, MVTEXTURE *tex);
	inline void setShader(MVSHADER *shader)							
	{ 
         #ifdef MV_CAVE
	      if (shader)
		      MVSETTINGS::sendValue(MV_MATERIAL_SETSHADER, id, shader->getId());
	      else
		      MVSETTINGS::sendValue(MV_MATERIAL_SETSHADER, id, 0);
         #endif			      
	      this->shader = shader; 
	} 
	inline MVCOLOR4 getAmbient() const									{ return ambient; }
	inline void setSkinning(bool s)                             { skinned = s; }
	inline MVCOLOR4 getDiffuse() const									{ return diffuse; }
	inline MVCOLOR4 getSpecular() const								   { return specular; }
	inline MVCOLOR4 getEmission() const   							   { return emission; }
	inline float getTransparency() const								{ return transparency; }
	inline float getShininess() const									{ return shininess; }		
	inline bool isTransparent() const									{ return (transparency != 1.0f); }
	inline bool isTextured() const                              { return (texture[0] != NULL); }
	inline bool isSkinned() const                               { return skinned; }
	inline MVTEXTURE *getTexture(unsigned c)                    { if (c<MV_TEXTURE_MAXNUMBEROFCHANNELS) return texture[c]; else return NULL; }
	inline MVSHADER *getShader()                                { return shader; }
	
	// Management:
	static bool cleanUp();
	static int getNumberOfMaterials();
	static MVMATERIAL *findByName(char *name);
	static MVMATERIAL *findById(MVID id);	
	static MVMATERIAL *getFirst()                               { return firstMaterial; }
	MVMATERIAL *getNext()                                       { return next; }
	MVMATERIAL *getPrevious()                                   { return prev; }
	
	// Rendering:
	static void reset()														{ currentMaterial = NULL; }
	bool render(void *data);
	
//////////////
	private: //
//////////////

	// Static:
	static MVMATERIAL *firstMaterial;
	static MVMATERIAL *currentMaterial;
	
	// Data:		
	MVID id;
	char name[MV_MAXNUMBEROFCHARS];			
	int timeStamp, references;	
	MVCOLOR4 ambient, diffuse, specular, emission;
	float transparency, shininess;	
	MVTEXTURE *texture[MV_TEXTURE_MAXNUMBEROFCHANNELS];	
	MVSHADER *shader;
	bool skinned;
	
	// Lists:
	MVMATERIAL *prev, *next;	
} MVMATERIAL; // Material
