//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_mesh.H					  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Limits:
	#define MV_MAXNUMBEROFBONESPERSKELETON 100 // Remember to update shader values
	#ifdef MV_PDA
	   #define MV_MAXNUMBEROFSKELETONS 16
   #else
      #define MV_MAXNUMBEROFSKELETONS 256
   #endif      	   



/////////////////
// BONE STRUCT //
/////////////////

typedef struct MVBONE
{
   char name[MV_MAXNUMBEROFCHARS];
   MVNODE *node;
   MVMATRIX baseMatrix;
} MVBONE;

typedef struct MVSKELETON
{
	MVMATRIX bone[MV_MAXNUMBEROFBONESPERSKELETON];
	MVNODE *node;	
} MVSKELETON;



//////////////////
// CLASS MVMESH //
//////////////////

//////////////////////////////////////////////////////
typedef class MV_API MVMESH : public MVNODE // Mesh //
//////////////////////////////////////////////////////
{   
	friend class MVELEMENT;
	friend class MVANIM;
	
/////////////
	public: //	
/////////////

	// Cons/dest:
	MVMESH();
	~MVMESH();		
	static bool init();
	static bool free();
	
	// Settings:
	void setMaterial(MVMATERIAL *mat);
	inline void setShadowCaster(bool s)        { shadowCaster = s; }
	inline void setDisplayStatus(bool v)       
   { 
      #ifdef MV_CAVE
         MVSETTINGS::sendValue(MV_MESH_SETDISPLAYSTATUS, id, v);
      #endif	
      hidden = !v; 
   }
	inline float getRadius()						 { return radius; }	
	inline MVVECTOR3 getMinimum()              { return minimum; }
	inline MVVECTOR3 getMaximum()              { return maximum; }
	inline MVMATERIAL *getMaterial()			    { return material; }
	inline int getNumberOfBones()              { return numberOfBonesInSkeleton; }
	inline bool getShadowCaster()              { return shadowCaster; }	
   inline bool isClosed()                     { return closed; }   
   inline bool isHidden()                     { return hidden; }
   inline bool isSkinned()                    { return numberOfBones > 0; }
   inline MVWORD getNumberOfVertices()        { return numberOfVertices; }
   inline MVWORD getNumberOfUVs()             { return numberOfUVs; }
   inline MVWORD getNumberOfFaces()           { return numberOfFaces; }   
   inline const MVVECTOR3 *getVertexPtr()           { return vertex; }
   inline const MVVECTOR3 *getVertexNormalPtr()     { return vertexNormal; }
   inline const MVUV *getUVPtr()                    { return uv; }
   inline const MVFACE *getFacePtr()                { return face; }
   inline const MVVECTOR3 *getFaceNormalPtr()       { return faceNormal; }   
	
	// Loading:	
	MVNODE *recursiveLoadXVE(MVBYTE *buffer, MVDWORD *position);	
	static MVNODE *load(char *buffer, long *position, bool obsoleteMVE = false);

	// Rendering:
	static inline void showNormals()           { displayNormals = true; }
   static inline void hideNormals()           { displayNormals = false; }
	static inline void reset() // Move me elsewhere
	{ 
			currentSkeleton = 0;
	      MVOPENGL::vertexArrayOff();
	      MVOPENGL::normalArrayOff();
	      MVOPENGL::colorArrayOff();
	      MVOPENGL::textureArrayOff();
#ifndef MV_PDA
	      if (MVOPENGL::glClientActiveTextureARB)
			      MVOPENGL::glClientActiveTextureARB(GL_TEXTURE0_ARB);							
#endif
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
#ifndef MV_PDA
			if (MVOPENGL::glClientActiveTextureARB)
      	      MVOPENGL::glClientActiveTextureARB(GL_TEXTURE1_ARB);							
#endif
		   glDisableClientState(GL_TEXTURE_COORD_ARRAY);
#ifndef MV_PDA
		  if (MVOPENGL::useVBO())
				MVOPENGL::glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, NULL);
#endif		   
	}		
	bool resetBones(MVNODE *root);
	bool animate(unsigned int skeleton);
	bool render(void *data);
	
//////////////
	private: //
//////////////

	// Data:   	
   float radius;
   MVVECTOR3 minimum, maximum;
	MVMATERIAL *material;	
   bool shadowCaster, closed, hidden;
	
	// Geometry:
   MVWORD numberOfVertices, numberOfUVs, numberOfFaces, numberOfBones, numberOfBonesInSkeleton;
   MVBYTE *data;
   MVVECTOR3 *vertex, *vertexNormal, *faceNormal;   
   MVUV *uv;   
	MVFACE *face;		
	MVBYTE *index;
	float *weight, *floatIndex;	
	MVBONE *skeleton;
	MVVECTOR3 *animatedVertex, *animatedVertexNormal;	
	unsigned int vertexVBO, vertexNormalVBO, uvVBO, faceVBO, floatIndexVBO, weightVBO;	
	
	// Shared:
	static bool displayNormals;
	
	// Skeleton cache:
	static unsigned int currentSkeleton;
	static MVSKELETON *skeletonElement;
} MVMESH; // Mesh
