//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_node.H					  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	typedef enum MV_NODE_TYPE // Kind of node
	{
			MV_NODE_NODE = 0,
			MV_NODE_ROOT,		
			MV_NODE_LIGHT,						
			MV_NODE_MESH,	
			MV_NODE_CAMERA,			
			MV_NODE_EMITTER,
			MV_NODE_SKYDOME,
			MV_NODE_WATER,							
			MV_NODE_TERRAIN,	
			MV_NODE_HELPER,											
			MV_NODE_SOUND,	
			MV_NODE_BOX,
			MV_NODE_LABEL,
			MV_NODE_BUTTON,
			MV_NODE_WINDOW,
			MV_NODE_TEXTFIELD,
			MV_NODE_HSCROLLBAR,
			MV_NODE_VSCROLLBAR,
			MV_NODE_CUSTOM,
			MV_NODE_NUMBEROFTYPES,
	} MV_NODE_TYPE;
	
	typedef enum MV_EXTRA_TYPE // Kind of type
	{
	   MV_EXTRA_TEXTURE = MV_NODE_NUMBEROFTYPES,
	   MV_EXTRA_MATERIAL,
	   MV_EXTRA_SHADER,
	   MV_EXTRA_FBO,
	   MV_EXTRA_ANIM,
	   MV_EXTRA_VIDEO,
	   MV_EXTRA_NUMBEROFTYPES,
	} MV_EXTRA_TYPE;

   typedef enum MV_NODE_POSITIONING // Kind of positioning
   {
         MV_POSITION,
         MV_MATRIX,
   } MV_NODE_POSITIONING;



//////////////////
// CLASS MVNODE //
//////////////////

/////////////////////////////////////////////
typedef class MV_API MVNODE // Node object //
/////////////////////////////////////////////
{   
	friend class MVISIO;
	friend class MVELEMENT;
	friend class MVREMOTE;
	
/////////////
	public: //	
/////////////

	// Cons/dest:
	MVNODE();
	MVNODE(bool f);
	virtual ~MVNODE();		
	
	// Settings:
	inline void setName(char *name)			{ if (name && MV_VALIDSTRING(name)) strcpy(this->name, name); }
	inline void setAuxId(MVID i)				{ auxId = i; }   
	inline MVID getId()	const					{ return id; }
	inline MVID getAuxId() const				{ return auxId; }
	inline char *getName()     				{ return name; }	
	inline MV_NODE_TYPE getType() const		{ return type; }
	inline char *getTypeName()					{ return typeName[type]; }
	inline long getTimeStamp() const			{ return timeStamp; }	
	inline void setData(void *d)           { data = d; }
	inline void *getData() const           { return data; }
	inline bool isShared()                 { return shared; }
	inline void setSharedStatus(bool s)    { shared = s; }
	
	// Management:
	static bool cleanUp();
	static unsigned getNumberOfNodes();
	static MVNODE *findByName(char *name);
	MVNODE *findByNameInHierarchy(char *name);
	static MVNODE *findById(MVID id);
	MVNODE *findByIdInHierarchy(MVID id);
	static MVNODE *getFirstNode()          { return firstNode; }
	MVNODE *getNextNode()                  { return next; }
	MVNODE *getPreviousNode()              { return prev; }
	
	// Hierarchy:	
	bool isRoot();
	inline unsigned getNumberOfChildren() const	{ return numberOfChildren; }	
	MVNODE *getChild(unsigned c);
	inline MVNODE *getParent() const					{ return parent; }
	bool link(MVNODE *node);	
	bool unlink();
	bool remove();	

	// Positioning:	
   inline void setPositioning(MV_NODE_POSITIONING p)   
   {  
         #ifdef MV_CAVE
            MVSETTINGS::sendValue(MV_SETPOSITIONING, id, p);
         #endif
         positioning = p; 
   }
	inline void setPosition(float x, float y)				 
	{ 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector2(MV_SETPOSITION2D, id, x, y);
         #endif
	      position.set(x, y, position.z); 
	}		
	inline void setPosition(float x, float y, float z)  
	{  
	      #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_SETPOSITION3D, id, x, y, z);
         #endif
	      position.set(x, y, z); 
	}   	
	inline void setRotation(float x, float y)				 
	{ 
	      #ifdef MV_CAVE
	         MVSETTINGS::sendVector2(MV_SETROTATION2D, id, x, y);
	      #endif
	      rotation.set(x, y, rotation.z); 
	}
   inline void setRotation(float x, float y, float z)  
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_SETROTATION3D, id, x, y, z);            
         #endif
         rotation.set(x, y, z); 
   }
   inline void setOrigin(float x, float y)				 { origin.set(x, y, origin.z); }
	inline void setOrigin(float x, float y, float z)    { origin.set(x, y, z); }
	inline void setDirection(float x, float y)			
	{ 
			#ifdef MV_CAVE
				MVSETTINGS::sendVector2(MV_SETDIRECTION2D, id, x, y);
			#endif
			direction.set(x, y, direction.z); 
	}
	inline void setDirection(float x, float y, float z) 
	{ 
			#ifdef MV_CAVE
				MVSETTINGS::sendVector3(MV_SETDIRECTION3D, id, x, y, z);
			#endif
			direction.set(x, y, z); 
	}
	inline void setTarget(float x, float y)				 
	{ 
	      #ifdef MV_CAVE
	         MVSETTINGS::sendVector2(MV_SETTARGET2D, id, x, y);
	      #endif
	      target.set(x, y, target.z); 
   }
	inline void setTarget(float x, float y, float z)    
	{ 
	      #ifdef MV_CAVE
	         MVSETTINGS::sendVector3(MV_SETTARGET3D, id, x, y, z);
	      #endif
	      target.set(x, y, z); 
	}
	inline void setPosition(MVVECTOR3 v)					 
	{ 
	      #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_SETPOSITION3D, id, v.x, v.y, v.z);
         #endif
	      position = v; 
	}
   inline void setRotation(MVVECTOR3 v)				 	 
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_SETROTATION3D, id, v.x, v.y, v.z);
         #endif
         rotation = v; 
   }
	inline void setOrigin(MVVECTOR3 v)						 { origin = v; }
	inline void setDirection(MVVECTOR3 v)					 
	{ 
			#ifdef MV_CAVE
				MVSETTINGS::sendVector3(MV_SETDIRECTION3D, id, v.x, v.y, v.z);
			#endif
			direction = v; 
	}
	inline void setTarget(MVVECTOR3 v)						 
	{ 
	      #ifdef MV_CAVE
	         MVSETTINGS::sendVector3(MV_SETTARGET3D, id, v.x, v.y,v.z);
	      #endif
	      target = v; 
	}
	inline void setTargetNode(MVNODE *t)					 
	{ 
	      #ifdef MV_CAVE
	         if (t)
	               MVSETTINGS::sendValue(MV_SETTARGETNODE, id, t->getId());
	         else
	               MVSETTINGS::sendValue(MV_SETTARGETNODE, id, NULL);
         #endif	               
	      targetNode = t; 
   }
   inline void setMatrix(MVMATRIX m)                   
   {
         #ifdef MV_CAVE
            MVSETTINGS::sendMatrix(MV_SETMATRIX, id, m);
         #endif      
         matrix = m; 
   }
   inline MV_NODE_POSITIONING getPositioning()         { return positioning; }
	inline MVVECTOR3 getPosition()							 { return position; }
   inline MVVECTOR3 getRotation()							 { return rotation; }
	inline MVVECTOR3 getOrigin()								 { return origin; }
	inline MVVECTOR3 getDirection()							 { return direction; }
	inline MVVECTOR3 getTarget()								 { return target; }
	inline MVNODE *getTargetNode()							 { return targetNode; }
   inline MVMATRIX getMatrix()                         { return matrix; }   
   inline MVMATRIX *getCachedMatrix()						 { return &cachedMatrix; }
	inline void move(float x, float y)						 
	{  
	      #ifdef MV_CAVE
            MVSETTINGS::sendVector2(MV_MOVE2D, id, x, y);
         #endif
	      position.x+=x; position.y+=y; 
	}
	inline void move(float x, float y, float z)			 
	{ 
	      #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_MOVE3D, id, x, y, z);
         #endif
	      position.x+=x; position.y+=y; position.z+=z; 
	}
	inline void rotate(float x, float y)					 
	{ 
	      #ifdef MV_CAVE
            MVSETTINGS::sendVector2(MV_ROTATE2D, id, x, y);
         #endif
	      rotation.x+=x; rotation.y+=y; 
	}
   inline void rotate(float x, float y, float z)		 
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_ROTATE3D, id, x, y, z);
         #endif	
         rotation.x+=x; rotation.y+=y; rotation.z+=z; 
   }
   inline void move(MVVECTOR3 v)							 
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_MOVE3D, id, v.x, v.y, v.z);
         #endif	
         position += v; 
   }
   inline void rotate(MVVECTOR3 v)							 
   { 
         #ifdef MV_CAVE
            MVSETTINGS::sendVector3(MV_ROTATE3D, id, v.x, v.y, v.z);
         #endif	
         rotation += v; 
   }   
   MVMATRIX getFinalMatrix(char *finalNode = NULL);   
	
	// Loading:
	static MVNODE *load(char *buffer, long *position);
	
	// Rendering:	
	bool pass(bool recurse = true);
	virtual bool render(void *data = NULL) { return false; }
	
	// Debugging:
	static MVNODE *displayTreeRecursive(MVNODE *node, int spacing);
	static bool displayTree(MVNODE *node);
	static char typeName[MV_NODE_NUMBEROFTYPES][MV_MAXNUMBEROFCHARS];
	
////////////////
	protected: //
////////////////

	// Static:
	static MVNODE *firstNode;
	
	// Data:
	MVID id, auxId;	
	char name[MV_MAXNUMBEROFCHARS];
	MV_NODE_TYPE type;	
	long timeStamp;	
	bool shared;
	void *data;

	// Positioning:
   MV_NODE_POSITIONING positioning;
	MVVECTOR3 position, rotation, origin, direction, target;
	MVNODE *targetNode;
   MVMATRIX matrix;
   MVMATRIX cachedMatrix;
	
	// Chained list:
	MVNODE *prev, *next;	
	
	// Hierarchycal list:	
	unsigned numberOfChildren;	
	MVNODE *parent, **children;		
} MVNODE; // Node object
