//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_opengl.H	              //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////	
	
	// FBO 3D names:
	typedef enum MV_OPENGL_FBONAMES
	{
	      MV_OPENGL_FBO_FRAMEBUFFER1 = 0, 	// Float, main FBO		
			MV_OPENGL_FBO_FRAMEBUFFER2,		// float, alternate FBO	
			MV_OPENGL_FBO_FRAMEBUFFER3,      // depth component, shadow maps
			MV_OPENGL_FBO_FRAMEBUFFER4,		// float, POT for HDR luminance		
			MV_OPENGL_FBO_BLURMAIN1,
			MV_OPENGL_FBO_BLURMAIN2,
			MV_OPENGL_FBO_BLURHALF1,
			MV_OPENGL_FBO_BLURHALF2,
			MV_OPENGL_FBO_BLURQUARTER1,
			MV_OPENGL_FBO_BLURQUARTER2,
			MV_OPENGL_FBO_BLUREIGHTEN1,
			MV_OPENGL_FBO_BLUREIGHTEN2,
			MV_OPENGL_FBO_BLURSIXTEN1,
			MV_OPENGL_FBO_BLURSIXTEN2,
			MV_OPENGL_FBO_LAST,
	} MV_OPENGL_FBONAMES;	

	// Default:
	#define MV_OPENGL_DEFAULT_NEARPLANE			   1.0f
	#define MV_OPENGL_DEFAULT_FARPLANE           1024.0f
	#define MV_OPENGL_DEFAULT_ANGLEOFVIEW        45.0f;
	#define MV_OPENGL_DEFAULT_BACKGROUNDCOLOR    0.2f, 0.2f, 0.6f, 1.0f
	#define MV_OPENGL_DEFAULT_GLOBALAMBIENTLIGHT 0.2f, 0.2f, 0.2f, 1.0f	
	
	// Quality/type of rendering:
	typedef enum MV_OPENGL_PROFILE
	{			
			MV_OPENGL_FAST,
			MV_OPENGL_NICE,			
			MV_OPENGL_PASSTHROUGH, // Passthroughs must be at the end
			MV_OPENGL_PASSTHROUGHZPASS,
			MV_OPENGL_MAXNUMBEROFPROFILES,
	} MV_OPENGL_PROFILE;



////////////////////
// CLASS MVOPENGL //
////////////////////

///////////////////////////////////////////////////
typedef class MV_API MVOPENGL // OpenGL specific //
///////////////////////////////////////////////////
{   
	friend class MVISIO;

/////////////
	public: //	
/////////////		
	
	// Settings:
	static inline void setNearPlane(float n)												  
	{ 
	      #ifdef MV_CAVE	
	         int _n; memcpy(&_n, &n, sizeof(int));
	         MVSETTINGS::sendValue(MV_OPENGL_SETNEARPLANE, NULL, _n);         
	      #endif 
	      nearPlane = n;	      
   }	
	static inline void setFarPlane(float f)												  
	{ 
	      #ifdef MV_CAVE	   
	         int _f; memcpy(&_f, &f, sizeof(int));
	         MVSETTINGS::sendValue(MV_OPENGL_SETFARPLANE, NULL, _f);               
	      #endif
	      farPlane = f; 	      
	}	
	static inline void setAngleOfView(float a)											  { angleOfView = a; }
	static void setBackgroundColor(MVCOLOR4 c);	
	static inline void setBackgroundColor(float r, float g, float b, float a)    { setBackgroundColor(MVCOLOR4(r, g, b, a)); }	
	static void setGlobalAmbientLight(MVCOLOR4 c);	
	static inline void setGlobalAmbientLight(float r, float g, float b, float a) { setGlobalAmbientLight(MVCOLOR4(r, g, b, a)); }		
	static inline void setProfile(MV_OPENGL_PROFILE p)									  { currentProfile = p; }					  
	static inline float getNearPlane()													     { return nearPlane; }
	static inline float getFarPlane()													     { return farPlane; }
	static inline float getAngleOfView()													  { return angleOfView; }
	static inline MVCOLOR4 getBackgroundColor()											  { return backgroundColor; }
	static inline MVCOLOR4 getGlobalAmbientLight()									     { return globalAmbientLight; }	
	static inline float getAnisotropicLevel()												  { return anisotropicLevel; }	
	static inline MV_OPENGL_PROFILE getProfile()											  { return currentProfile; }	
	static bool resize(int x, int y, int sizeX, int sizeY);
	static void setPerspective(float FOV, float aspect, float zNear, float zFar);
	static bool setFrustum(float leftPlane, float rightPlane, float bottomPlane, float topPlane, float nearPlane, float farPlane, bool infinite = true);
	
	// Cache:
   static inline void cullingOn() 																	  { if (!culling)			     { glEnable(GL_CULL_FACE); culling = true; } }
	static inline void cullingOff()																	  { if (culling)				  { glDisable(GL_CULL_FACE); culling = false; } }					
	static inline bool isCulled()                                                      { return culling; }
	static inline void twoSideOn()                                                     { if (!twoSide)			     { glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0f); twoSide = true;  } }
	static inline void twoSideOff()                                                    { if (twoSide)             { glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 0.0f); twoSide = false; } }
	static inline bool isTwoSide()                                                     { return twoSide; } 
	static inline void blendingOn() 																	  { if (!blending)			  { glEnable(GL_BLEND); blending = true; } }
	static inline void blendingOff()																	  { if (blending)				  { glDisable(GL_BLEND); blending = false; } }				
	static inline bool isBlended()                                                     { return blending; }
	static inline void scissorOn() 																	  { if (!scissor)			     { glEnable(GL_SCISSOR_TEST); scissor = true; } }
	static inline void scissorOff()																	  { if (scissor)				  { glDisable(GL_SCISSOR_TEST); scissor = false; } }					   
	static inline void vertexArrayOn()																  { if (!vertexArrayStatus)  { glEnableClientState(GL_VERTEX_ARRAY); vertexArrayStatus = true; } }	
	static inline void vertexArrayOff()																  { if (vertexArrayStatus)   { glDisableClientState(GL_VERTEX_ARRAY); vertexArrayStatus = false; } }	
#ifndef MV_PDA
   static inline void indexArrayOn()																  { if (!indexArrayStatus)  { glEnableClientState(GL_INDEX_ARRAY); indexArrayStatus = true; } }	
	static inline void indexArrayOff()																  { if (indexArrayStatus)   { glDisableClientState(GL_INDEX_ARRAY); indexArrayStatus = false; } }	
#endif
	static inline void normalArrayOn()																  { if (!normalArrayStatus)  { glEnableClientState(GL_NORMAL_ARRAY); normalArrayStatus = true; } }	
	static inline void normalArrayOff()																  { if (normalArrayStatus)   { glDisableClientState(GL_NORMAL_ARRAY); normalArrayStatus = false; } }	
	static inline void colorArrayOn()																  { if (!colorArrayStatus)   { glEnableClientState(GL_COLOR_ARRAY); colorArrayStatus = true; } }	
	static inline void colorArrayOff()																  { if (colorArrayStatus)    { glDisableClientState(GL_COLOR_ARRAY); colorArrayStatus = false; } }	
	static inline void textureArrayOn()																  
	{ 
	      if (!textureArrayStatus) 
	      { 
	            #ifndef MV_PDA
	            if (MVOPENGL::glClientActiveTextureARB) 
	                  MVOPENGL::glClientActiveTextureARB(GL_TEXTURE0_ARB); 
               #endif	                  
               glEnableClientState(GL_TEXTURE_COORD_ARRAY); textureArrayStatus = true; 
         } 
   }	
	static inline void textureArrayOff()																  
	{ 
	      if (textureArrayStatus)  
	      { 
	            #ifndef MV_PDA
	            if (MVOPENGL::glClientActiveTextureARB) 
	                  MVOPENGL::glClientActiveTextureARB(GL_TEXTURE0_ARB); 
               #endif	                  
	            glDisableClientState(GL_TEXTURE_COORD_ARRAY); 
	            textureArrayStatus = false; 
	      } 
   }	
   
   // FBO:
   static inline MVFBO *getFbo(int c)                   
   { 
			if (c >=0 && c <= MV_OPENGL_FBO_LAST)
				return fbo[c]; 
			else
			{
					// MV_LOG("   [!] Invalid params @ MVOPENGL::getFBO3D\n");
					return NULL;
			}				
	}  
	
#ifndef MV_PDA
	// Multitexturing extension:
	static PFNGLMULTITEXCOORD2FARBPROC glMultiTexCoord2fARB;
	static PFNGLACTIVETEXTUREARBPROC	glActiveTextureARB;
	static PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;

	// VBO extension function pointers:
	static PFNGLGENBUFFERSARBPROC glGenBuffersARB;
	static PFNGLBINDBUFFERARBPROC glBindBufferARB;
	static PFNGLBUFFERDATAARBPROC glBufferDataARB;
	static PFNGLDELETEBUFFERSARBPROC glDeleteBuffersARB;	
	
	// Shaders:
	static PFNGLCREATESHADEROBJECTARBPROC glCreateShaderObjectARB;
	static PFNGLSHADERSOURCEARBPROC glShaderSourceARB;
	static PFNGLCOMPILESHADERARBPROC glCompileShaderARB;
	static PFNGLCREATEPROGRAMOBJECTARBPROC glCreateProgramObjectARB;
	static PFNGLATTACHOBJECTARBPROC glAttachObjectARB;
	static PFNGLLINKPROGRAMARBPROC glLinkProgramARB;
	static PFNGLUSEPROGRAMOBJECTARBPROC glUseProgramObjectARB;
	static PFNGLDELETEOBJECTARBPROC glDeleteObjectARB;
	static PFNGLDETACHOBJECTARBPROC glDetachObjectARB;
	static PFNGLGETOBJECTPARAMETERIVARBPROC glGetObjectParameterivARB;
	static PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocationARB;
	static PFNGLUNIFORM1IPROC glUniform1iARB;		
	static PFNGLUNIFORM1FPROC glUniform1fARB;
	static PFNGLUNIFORM3FPROC glUniform3fARB;
	static PFNGLUNIFORM4FPROC glUniform4fARB;
	static PFNGLUNIFORM3FVPROC glUniform3fvARB;
	static PFNGLUNIFORM4FVPROC glUniform4fvARB;
	static PFNGLUNIFORMMATRIX4FVPROC glUniformMatrix4fvARB;
	static PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointerARB;
	static PFNGLENABLEVERTEXATTRIBARRAYPROC glEnableVertexAttribArrayARB;
	static PFNGLDISABLEVERTEXATTRIBARRAYPROC glDisableVertexAttribArrayARB;
	static PFNGLGETATTRIBLOCATIONPROC glGetAttribLocationARB;  
	
	// Frame buffer:
	static PFNGLGENFRAMEBUFFERSEXTPROC glGenFramebuffersEXT;
	static PFNGLDELETEFRAMEBUFFERSEXTPROC glDeleteFramebuffersEXT;
	static PFNGLBINDFRAMEBUFFEREXTPROC glBindFramebufferEXT;
	static PFNGLFRAMEBUFFERTEXTURE2DEXTPROC glFramebufferTexture2DEXT;
	static PFNGLGENRENDERBUFFERSEXTPROC glGenRenderbuffersEXT;
	static PFNGLDELETERENDERBUFFERSEXTPROC glDeleteRenderbuffersEXT;
	static PFNGLBINDRENDERBUFFEREXTPROC glBindRenderbufferEXT;
	static PFNGLRENDERBUFFERSTORAGEEXTPROC glRenderbufferStorageEXT;
	static PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC glFramebufferRenderbufferEXT;
	static PFNGLGENERATEMIPMAPEXTPROC glGenerateMipmapEXT;
	static PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC glCheckFramebufferStatusEXT;	

   // Occlusion query:
	static PFNGLGENQUERIESARBPROC glGenQueriesARB;
	static PFNGLDELETEQUERIESARBPROC glDeleteQueriesARB;
	static PFNGLISQUERYARBPROC glIsQueryARB;
	static PFNGLBEGINQUERYARBPROC glBeginQueryARB;
	static PFNGLENDQUERYARBPROC glEndQueryARB;
	static PFNGLGETQUERYIVARBPROC glGetQueryivARB;
	static PFNGLGETQUERYOBJECTIVARBPROC glGetQueryObjectivARB;
	static PFNGLGETQUERYOBJECTUIVARBPROC glGetQueryObjectuivARB;
#endif 
		
   // Extension flags:   
   static bool useVBO()                                       { return VBOFlag; }
   static bool useAnisotropic()                               { return anisotropicFlag; }
   static bool useFBO()                                       { return FBOFlag; }   
   static bool useMultitexturing()                            { return multitextureFlag; }
   static bool useShaders()                                   { return shadersFlag; }
   static bool useOcclusionQuery()							        { return OcclusionQueryFlag; }
		
//////////////
	private: //
//////////////

	// Status:
	static bool active;	
	
	// Settings:
	static float nearPlane;
	static float farPlane;
	static float angleOfView;
	static MVCOLOR4 backgroundColor;	
	static MVCOLOR4 globalAmbientLight;
	static float anisotropicLevel;	
	static MV_OPENGL_PROFILE currentProfile;
	
	// Flag cache:
	static bool blending;
	static bool scissor;
	static bool twoSide;
	static bool culling;
	static bool vertexArrayStatus;
   static bool indexArrayStatus;
	static bool normalArrayStatus;
	static bool colorArrayStatus;
	static bool textureArrayStatus;	
	
	// Flag extensions:
	static bool multitextureFlag;
	static bool VBOFlag;
	static bool shadersFlag;
	static bool anisotropicFlag;
	static bool FBOFlag;
   static bool OcclusionQueryFlag;
	
	// Framebuffer:
	static MVFBO *fbo[MV_OPENGL_FBO_LAST];
	
	// Init/free:
	static inline bool isActive() { return active; }
	static bool init();
	static bool reset();	
	static bool free();	
} MVOPENGL; // OpenGL specific
