//////////////////////////////////////////////
//                                          //
// project  : mVISIO v1.6a                  //
// file     : mvisio_shader.H					  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Limits:
	#define MV_MAXNUMBEROFSHADERPARAMS 28
	


////////////////////////////////////////////////////
typedef class MV_API MVSHADER // Shader interface //
////////////////////////////////////////////////////
{   
	friend class MVISIO;

/////////////
	public: //	
/////////////	

   // Default shaders:	
	static MVSHADER *pplDirect;	
	static MVSHADER *pplOmni;	
	static MVSHADER *pplSpot;		
	static MVSHADER *pplDirectA;
	static MVSHADER *pplOmniA;
	static MVSHADER *pplSpotA;	
	static MVSHADER *pplDirectT;	
	static MVSHADER *pplOmniT;	
   static MVSHADER *pplSpotT;   
   static MVSHADER *pplDirectTA;
   static MVSHADER *pplOmniTA;
   static MVSHADER *pplSpotTA;   
	static MVSHADER *pplSpotS;		
	static MVSHADER *pplSpotSA;	
	static MVSHADER *pplSpotTS;	
	static MVSHADER *pplSpotTSA;	
	static MVSHADER *passthrough;
	static MVSHADER *passthroughA;
	static MVSHADER *passthroughZPass;	
	static MVSHADER *passthroughZPassA;	
	static MVSHADER *bloomBright;
	static MVSHADER *bloomToneMapping;
	static MVSHADER *bloomForceAlpha;
	static MVSHADER *bloomCombine;
	static MVSHADER *bloomV;	
	static MVSHADER *bloomH;	
	static MVSHADER *gaussian;		
	static MVSHADER *emitter;	
	static MVSHADER *dofCombine;	
	static MVSHADER *dof;	
	static MVSHADER *skydome;
	static MVSHADER *water;
   static MVSHADER *waterS;
	static MVSHADER *terrainComplex;
	static MVSHADER *terrainSimple;

	// Cons/dest:
	MVSHADER();
	~MVSHADER();		
   
   // Settings:
   inline MVID getId() const												{ return id; }
	inline void setName(char *n)											
	{ 
	      #ifdef MV_CAVE
            MVSETTINGS::sendString(MV_SHADER_SETNAME, id, n);   
         #endif
	      if (n && MV_VALIDSTRING(n)) strcpy(name, n); 
	}
	inline char *getName()													{ return name; }
	inline char *getVSFilename()											{ return VSfilename; }
	inline char *getPSFilename()											{ return PSfilename; }
	inline bool isOperative()                                   { return VSoperative && PSoperative; }
	inline void forceDeactivation()                             { VSoperative = PSoperative = false; }
	
	// Shader codepath:
	static bool useSkinning()                                   { return skinningFlag; }          
	static bool usePPL()                                        { return pplFlag; }
	static bool useBloom()                                      { return bloomFlag; }
	static bool useShadows()                                    { return shadowsFlag; }
	static bool usePassthrough()                                { return passthroughFlag; }
	static bool useEmitter()                                    { return emitterFlag; }
	static bool useDOF()														{ return dofFlag; }
	static bool useSkydome()                                    { return skydomeFlag; }
	static bool useWater()                                      { return waterFlag; }
	static bool useTerrain()                                    { return terrainFlag; }
   
   // Management:
   static bool cleanUp();
   static int getNumberOfShaders();
	static MVSHADER *findByName(char *name);
	static MVSHADER *findById(MVID id);  	
	static MVSHADER *getFirst()                                 { return firstShader; }
	MVSHADER *getNext()                                         { return next; }
	MVSHADER *getPrevious()                                     { return prev; }
	
   // Loading:
   static void setPath(char *path)										{ if (path) strcpy(shaderPath, path); else strcpy(shaderPath, ""); }
	static char *getPath() 													{ return shaderPath; }
   bool loadVertexShader(char *filename, char *code = NULL);
   bool loadPixelShader(char *filename, char *code = NULL);
   bool load(char *vertexFilename, char *pixelFilename, bool justBind = false);      
   
   // Parameters:
   inline GLuint getParam(unsigned p) const                    { if (p<MV_MAXNUMBEROFSHADERPARAMS) return param[p]; else return MV_MAXNUMBEROFSHADERPARAMS; }
   bool bindParam(unsigned p, char *paramStr, bool attrib = false);
  
   // Rendering:
   static void reset()														{ currentShader = NULL; disable(); }
   static bool enable();
	static bool disable();   	
	static bool bindTexture(unsigned level);		
	static bool bindBones(unsigned bonesInSkeleton, float *quaternion, float *translation, unsigned int floatIndexVBO, unsigned int weightVBO);
	static bool unbindBones();
	static bool bindSizes(int width, int height);
	static bool bindLuminanceAndExposure(float luminance, float exposure);
	static bool bindNearAndFarPlane(float nearPlane, float farPlane);	
	static bool bindFocusAndRange(float focus, float range);
   bool render(void *data = NULL);
   
//////////////
	private: //
//////////////

	// Status:
	static bool active;
	
	// Static:
	static MVSHADER *firstShader;
	static MVSHADER *currentShader;		 
	
   // Init/free:
	static inline bool isActive() { return active; }
	static bool init();
	static bool free();
	
	// Path:
	static char shaderPath[MV_MAXNUMBEROFCHARS];
	
	// Data:
	MVID id;	
	char name[MV_MAXNUMBEROFCHARS];
	char VSfilename[MV_MAXNUMBEROFCHARS], PSfilename[MV_MAXNUMBEROFCHARS];	 
	bool VSoperative, PSoperative;
#ifndef MV_PDA
	GLhandleARB VSshader, PSshader, shader;  
#endif  
   GLuint param[MV_MAXNUMBEROFSHADERPARAMS];
  	
	// Lists:
	MVSHADER *prev, *next;
	
	// Codepath flags:
	static bool bloomFlag;
	static bool pplFlag;
	static bool shadowsFlag;
	static bool skinningFlag;	
	static bool passthroughFlag;
	static bool emitterFlag;
	static bool dofFlag;
	static bool skydomeFlag;
	static bool waterFlag;
	static bool terrainFlag;
} MVSHADER; // Shader interface
