//////////////////////////////////////////////
//                                          //
// project  : mVISIO v0.1a                  //
// file     : mvisio_texture.H				  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005                 //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

	// Limits:	
	#define MV_TEXTURE_MAXNUMBEROFCHANNELS 16
#ifndef MV_PDA
	#define MV_MAXTEXTURESIZE 4096	
#else
   #ifdef MV_PDA_SOFTWARE
	   #define MV_MAXTEXTURESIZE 512
   #else
      #define MV_MAXTEXTURESIZE 256
   #endif      	   
#endif   
	
	// Default:	
	#define MV_TEXTURE_DEFAULT_TRILINEAR true
#ifdef MV_PDA
	#ifdef MV_PDA_SOFTWARE
		#undef MV_TEXTURE_DEFAULT_TRILINEAR
		#define MV_TEXTURE_DEFAULT_TRILINEAR	false
	#endif
#endif
	#define MV_TEXTURE_DEFAULT_MIPMAPPING  true
	#define MV_TEXTURE_MPGBUFFERSIZE 4096
	
	// Type:
	typedef enum MV_TEXTURE_TYPE
	{
		MV_TEXTURE_24BIT = 3,
		MV_TEXTURE_32BIT = 4,
		MV_TEXTURE_DEPTH = 5,
		MV_TEXTURE_FLOAT = 6,
		MV_TEXTURE_CUBE  = 7,
	} MV_TEXTURE_TYPE;		
	
	typedef enum MV_TEXTURE_CUBE_TYPE
	{
		MV_TEXTURE_CUBE_FRONT,
		MV_TEXTURE_CUBE_RIGHT,
		MV_TEXTURE_CUBE_BACK,
		MV_TEXTURE_CUBE_LEFT,
		MV_TEXTURE_CUBE_UP,
		MV_TEXTURE_CUBE_BOTTOM,
	} MV_TEXTURE_CUBE_TYPE;
	

	
/////////////////////
// CLASS MVTEXTURE //
/////////////////////


////////////////////////////////////////////
typedef class MV_API MVTEXTURE // Texture //
////////////////////////////////////////////
{   
/////////////
	public: //	
/////////////

	// Cons/dest:
	MVTEXTURE();
	~MVTEXTURE();		
	
	// Settings:	
	inline void incReferences()											{ references++; }
	inline void decReferences()											{ references--; }
	inline int  getReferences()											{ return references; }		
	bool setFilter(bool _trilinear, bool _mipMapping, bool _clamp = false);	
	inline void setCubeType(unsigned ct)								{ cubeType = ct; }
	inline MVID getId()														{ return id; }	
	inline GLuint getGLId()                                     { return glId; }
	inline char *getFilename()												{ return filename; }											
	inline unsigned getSizeX()												{ return sizeX; }
	inline unsigned getSizeY()												{ return sizeY; }
	inline unsigned char *getBitmap()                           { return bitmap; }
	inline long getTimeStamp()												{ return timeStamp; }
	inline bool isLoaded()													{ return glId != 0; }		
	inline bool isTrilinear()												{ return trilinear; }
	inline bool isMipmapped()												{ return mipMapping; }	
	inline bool isClamped()                                     { return clamp; }
	
	// Management:	
	static bool cleanUp();	
	static unsigned getNumberOfTextures();
	static inline int getUsedMemory()									{ return usedMemory; }		
	static MVTEXTURE *findByFilename(char *name);
	static MVTEXTURE *findById(MVID id);
	static MVTEXTURE *getFirst()                                { return firstTexture; }
	MVTEXTURE *getNext()                                        { return next; }
	MVTEXTURE *getPrevious()                                    { return prev; }

	// Loading:
	static void setPath(char *path)										{ if (path) strcpy(texturePath, path); else strcpy(texturePath, ""); }
	static char *getPath() 													{ return texturePath; }	
	bool create(unsigned sizeX, unsigned sizeY, unsigned type);
	bool load(char *filename);	
	bool reload();
	bool modify(char *data, bool copyMemory = true);
	bool update();
	bool unload(bool full = true);
	static bool garbageCollect(unsigned long bytes);		
	
	// Rendering:	
	static bool texturingOn(unsigned channel = MV_TEXTURE_MAXNUMBEROFCHANNELS);
	static bool texturingOff(unsigned channel = MV_TEXTURE_MAXNUMBEROFCHANNELS);
	bool render(void *data = NULL);

//////////////
	private: //
//////////////

	// List:
	static MVTEXTURE *firstTexture;
	static unsigned long usedMemory;
	static bool channelActive[MV_TEXTURE_MAXNUMBEROFCHANNELS];
	static MVTEXTURE *channelCache[MV_TEXTURE_MAXNUMBEROFCHANNELS];	
	
	// Path:
	static char texturePath[MV_MAXNUMBEROFCHARS];
	
	// Data:		
	MVID id;	
	char filename[MV_MAXNUMBEROFCHARS];	
	long timeStamp;	
	int  references;
	unsigned char *bitmap;
	unsigned sizeX, sizeY, memory, type, cubeType;	
	int viewport[4];			  
	GLuint glId;   
	bool trilinear, mipMapping, clamp, _mipMapping;	
	MVTEXTURE *prev, *next;		
} MVTEXTURE; // Texture
