//////////////////////////////////////////////
//                                          //
// project  : mVISIO v1.6a                  //
// file     : mvisio_video.H					  //
// author   : Achille Peternier             // 
// copyright: EPFL (C) 2005-2006            //
//												        //
//////////////////////////////////////////////



/////////////
// #DEFINE //
/////////////

   // Buffering:
   #define MV_MPGBUFFERSIZE 4096

   // Status:
   typedef enum MV_VIDEO_STATUS // Current status
	{			
			MV_VIDEO_STOPPED,
			MV_VIDEO_PLAYING,
			MV_VIDEO_PLAYINGLOOP,
			MV_VIDEO_REWIND,			
   } MV_VIDEO_STATUS;			



///////////////////
// CLASS MVVIDEO //
///////////////////

////////////////////////////////////////////
typedef class MV_API MVVIDEO // Animation //
////////////////////////////////////////////
{   	
/////////////
	public: //	
/////////////

	// Cons/dest:
	MVVIDEO();
	~MVVIDEO();			
	
	// Management:	
	static bool cleanUp();	
	static unsigned getNumberOfVideos();	
	static MVVIDEO *findByFilename(char *name);
	static MVVIDEO *findById(MVID id);	
	
	// Loading:
	static void setMPGPath(char *path)								{ if (path) strcpy(mpgPath, path); else strcpy(mpgPath, ""); }
	static char *getMPGPath() 											{ return mpgPath; }
	bool load(char *filename);
	inline void setTexture(MVTEXTURE *t)                     
	{ 
	      #ifdef MV_CAVE
	         int texId = 0;
            if (t)
               texId = t->getId();      
            MVSETTINGS::sendValue(MV_VIDEO_SETTEXTURE, id, texId);         
         #endif 	
	      texture = t; 
	}
	inline MVTEXTURE *getTexture()                           { return texture; }
	
	// Rendering:
	inline void play()                                       
	{ 
         #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_VIDEO_PLAY, id);
         #endif	
	      status = MV_VIDEO_PLAYING; 
   }
	inline void stop()                                       
	{ 
	      #ifdef MV_CAVE
            MVSETTINGS::sendCommand(MV_VIDEO_STOP, id);
         #endif
	      status = MV_VIDEO_STOPPED; 
	}
	inline void playLoop()                                   
	{ 
         #ifdef MV_CAVE
	         MVSETTINGS::sendCommand(MV_VIDEO_PLAYLOOP, id);
         #endif	
	      status = MV_VIDEO_PLAYINGLOOP; 
	}
	inline void rewind()                                    
	{ 
         #ifdef MV_CAVE
	         MVSETTINGS::sendCommand(MV_VIDEO_CMDREWIND, id);
         #endif		
         status = MV_VIDEO_REWIND; 
   }
	inline MV_VIDEO_STATUS getStatus()                       { return status; }
	bool update(unsigned int times = 1);
	
	
//////////////
	private: //
//////////////

   // Data: 
	MVID id;
	char filename[MV_MAXNUMBEROFCHARS];		
	MVTEXTURE *texture;
	
	// List:
	static MVVIDEO *firstVideo;
	MVVIDEO *prev, *next;  	   
	
	// Path:
	static char mpgPath[MV_MAXNUMBEROFCHARS];	
	
	// MPeg related (to hide dependices to mpeg2.h):
	MV_VIDEO_STATUS status;
	void *intStruct;
} MVVIDEO; // Video
