//
// VRcamera, handled by a joypad
// Author: Achille Peternier, VRLab 2006
// Version: 0.2
//
#ifndef _VRCAMERA_H
#define _VRCAMERA_H
#include <math.h>
#include <sdl/sdl.h>

// Beware, SDL.h and MVisio.h must have been included before this file

   // HARDCODED calibration (yes, that sucks, so if you have problems with that pls delete this file and *%*#*+%, ok?
   #define VRC_AXISX_CENTER -1
   #define VRC_AXISX_MIN    -32768
   #define VRC_AXISX_MAX    32767
   
   #define VRC_AXISY_CENTER -257
   #define VRC_AXISY_MIN    -32768
   #define VRC_AXISY_MAX    32767
   
   #define VRC_AXISZ_CENTER -257
   #define VRC_AXISZ_MIN    -32768  
   #define VRC_AXISZ_MAX    32767
   
   #define VRC_AXISW_CENTER -1
   #define VRC_AXISW_MIN    -32768
   #define VRC_AXISW_MAX    32767   
   
   #define VRC_INTTOLERANCE      512*12   // Dead zone around the center of the joypad
   #define VRC_FLOATTOLERANCE    0.001f
   #define VRC_CONSTRAINTANGLEX  70.0f   
   #define VRC_TOLERANCERATIO    (1.0f-(((float)VRC_INTTOLERANCE*2.0f) / (VRC_AXISX_MAX+abs(VRC_AXISX_MIN))))
   
   // Speed:
   #define VRC_ROTATIONSPEED        240.0f  // Degrees per second
   #define VRC_TRANSLATIONSPEED     10.0f   // Units per second
   #define VRC_LINEARITY            1.25f   // Use 1.0f to linear, 2.0f to exponential   

   // Global joystick:
   SDL_Joystick *joystick = NULL;
   MVCAMERA *VRcamera = NULL;   
   float VRtranslationSpeed = 1.0f;
   float VRrotationSpeed = 1.0f;
   bool VRuseKeyboard = false;
   bool VRkeyboardActive = true;
   MVVECTOR3 VRprevious, VRacceleration;
   
   // Methods:
   void VRC_useKeyboard();
   void VRC_useJoypad();
   bool VRC_init(MVCAMERA *camera, int joyID = 0);
   bool VRC_free();
   bool VRC_update(float fpsFactor = 0.05f);
   bool VRC_update(int mouseRX, int mouseRY, float fpsFactor = 0.05f);
   inline bool VRC_isKeyboardActive()   { return VRkeyboardActive; }
   


////////////////////
// IMPLEMENTATION //
////////////////////

   ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	void VRC_useKeyboard()
	/*
	** ?: activate camera by keyboard
	*/
{
   VRuseKeyboard = true;
   VRkeyboardActive = true;
   SDL_ShowCursor(0);
   SDL_WM_GrabInput(SDL_GRAB_ON);
   SDL_GetRelativeMouseState(NULL, NULL);
}
         
         
   ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	void VRC_useJoypad()
	/*
	** ?: activate camera by joypad
	*/
{
   VRuseKeyboard = false;
   SDL_ShowCursor(1);
   SDL_WM_GrabInput(SDL_GRAB_OFF);
}
   
         
   ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   bool VRC_init(MVCAMERA *camera, int joyID)
   /*
   ** ?: initialize the system
   ** !: call this AFTER SDL_INIT
   ** <: TF
   */
{
   SDL_InitSubSystem(SDL_INIT_JOYSTICK);
   
   // Check for joystick:
   if (SDL_NumJoysticks()>0)
   {
      // Open joystick:
      if (joyID <= SDL_NumJoysticks())
            joystick = SDL_JoystickOpen(joyID);
      else
      {
            #ifdef _DEBUG
               printf("VRC: error on joyID (VRC_init)\n");            
            #endif   
            SDL_QuitSubSystem(SDL_INIT_JOYSTICK);            
            return false;
      }            
   }
   else // Use keyboard by default:
   {
         #ifdef _DEBUG
            printf("VRC: joystick not detected, using keyboard (VRC_init)\n");
         #endif
         VRC_useKeyboard();
         VRcamera = camera;   
         VRprevious = camera->getPosition();      
         SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
         return false;            
   }   
  
   if (joystick)
   {
         #ifdef _DEBUG
            printf("Opened Joystick 0\n");
            printf("Name: %s\n", SDL_JoystickName(joyID));
            printf("Number of Axes: %d\n", SDL_JoystickNumAxes(joystick));
            printf("Number of Buttons: %d\n", SDL_JoystickNumButtons(joystick));
            printf("Number of Balls: %d\n", SDL_JoystickNumBalls(joystick));
         #endif         
   }
   else
   {
         #ifdef _DEBUG   
            printf("Couldn't open Joystick %d\n", joyID);
         #endif
         SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
         return false;         
   }
   
   // Done:   
   VRcamera = camera;
   VRprevious = camera->getPosition();
   return true;
}  


   ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   bool VRC_free()
   /*
   ** ?: free the system   
   ** <: TF
   */
{
   // Do it?
   if (joystick)
   {
	   SDL_JoystickClose(joystick);
	   joystick = NULL;
	   SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
	   VRcamera = NULL;
	   return true;	   
	}
	
	// Not needed:
	return false;
}


   ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   bool VRC_update(float fpsFactor)
   /*
   ** ?: with autoaquired values:
   */
   {
      int mouseRX = 0, mouseRY = 0;
      SDL_GetRelativeMouseState(&mouseRX, &mouseRY);       
      return VRC_update(mouseRX, mouseRY, fpsFactor);
   }
   
   bool VRC_update(int mouseRX, int mouseRY, float fpsFactor)
   /*
   ** ?: update camera position and rotation
   ** <: TF
   */
{
   // Check:
   if ((joystick==NULL && VRuseKeyboard==false) || VRcamera==NULL)
         return false;


   ////////////////         
   // Use keyboard:
   if (VRuseKeyboard)
   {
         // Keyboard movements:    
			Uint8 *array;
			int arraySize;
			array = SDL_GetKeyState(&arraySize);
			
         if (VRkeyboardActive)
         {
               if (!(array[SDLK_RCTRL] || array[SDLK_LCTRL]) && array[SDLK_k])
               {
                     SDL_ShowCursor(1);
                     SDL_WM_GrabInput(SDL_GRAB_OFF);
                     VRkeyboardActive = false;
               }
               
               // Mouse rotations:         
               int mouseDX = mouseRX, mouseDY = mouseRY, buttonState;
               buttonState = SDL_GetRelativeMouseState(NULL, NULL); 
         
               // Delinearize rotations:
               float _mX = pow(abs(((float) mouseDX)/(VRC_LINEARITY * 12.0f)), VRC_LINEARITY);
               if (mouseDX < 0) _mX = -_mX;
               float _mY = pow(abs(((float) mouseDY)/(VRC_LINEARITY * 12.0f)), VRC_LINEARITY);
               if (mouseDY < 0) _mY = -_mY;
         
               float mouseRotation = -1.0f * (VRC_ROTATIONSPEED/5.0f) * fpsFactor;
               VRcamera->rotate(_mY*mouseRotation, _mX*mouseRotation, 0.0f);
			
			      // Speed:
			      if (array[SDLK_1]) VRtranslationSpeed = 1.0f;
			      if (array[SDLK_2]) VRtranslationSpeed = 2.0f;
			      if (array[SDLK_3]) VRtranslationSpeed = 4.0f;
			      if (array[SDLK_4]) VRtranslationSpeed = 8.0f;
   		
			      // Forward:
			      float speedX=0.0f, speedY=0.0f, speedZ=0.0f;
			      if (array[SDLK_UP]    || array[SDLK_w] || buttonState & SDL_BUTTON(1)) speedZ -= VRC_TOLERANCERATIO;         
               if (array[SDLK_DOWN]  || array[SDLK_s] || buttonState & SDL_BUTTON(3)) speedZ += VRC_TOLERANCERATIO;
               if (array[SDLK_RIGHT] || array[SDLK_d] || buttonState & SDL_BUTTON(4)) speedX += VRC_TOLERANCERATIO;         
               if (array[SDLK_LEFT]  || array[SDLK_a] || buttonState & SDL_BUTTON(5)) speedX -= VRC_TOLERANCERATIO;               
               if (array[SDLK_e]) speedY += VRC_TOLERANCERATIO;
               if (array[SDLK_q]) speedY -= VRC_TOLERANCERATIO;
         
               // Run:
               float runFactor = 1.0f;
               if (array[SDLK_RSHIFT] || array[SDLK_LSHIFT] || buttonState & SDL_BUTTON(2)) runFactor = 2.0f;
         
               // Apply:                              
               VRcamera->move(0.0f, runFactor * VRtranslationSpeed * speedY * VRC_TRANSLATIONSPEED * fpsFactor, 0.0f);
               VRcamera->moveDoom(runFactor * VRtranslationSpeed * speedX * VRC_TRANSLATIONSPEED * fpsFactor, 
                                  runFactor * VRtranslationSpeed * speedY * VRC_TRANSLATIONSPEED * fpsFactor,               
                                  runFactor * VRtranslationSpeed * speedZ * VRC_TRANSLATIONSPEED * fpsFactor);
         }      
         else
               if ((array[SDLK_RCTRL] || array[SDLK_LCTRL]) && array[SDLK_k])
               {
                     SDL_ShowCursor(0);
                     SDL_WM_GrabInput(SDL_GRAB_ON);
                     SDL_GetRelativeMouseState(NULL, NULL);
                     VRkeyboardActive = true;
               }            
   }      
  
  
   //////////////
   // Use joypad:
   else
   {
         // Update values:       
         SDL_JoystickUpdate();  
         int axisX = SDL_JoystickGetAxis(joystick, 0);
         int axisY = SDL_JoystickGetAxis(joystick, 1);
         int axisZ = SDL_JoystickGetAxis(joystick, 2);
         int axisW = SDL_JoystickGetAxis(joystick, 3);         
   
         // Round to center:
         if (abs(axisX - VRC_AXISX_CENTER) < VRC_INTTOLERANCE) axisX = 0;
         if (abs(axisY - VRC_AXISY_CENTER) < VRC_INTTOLERANCE) axisY = 0;
         if (abs(axisZ - VRC_AXISZ_CENTER) < VRC_INTTOLERANCE) axisZ = 0;
         if (abs(axisW - VRC_AXISW_CENTER) < VRC_INTTOLERANCE) axisW = 0;
         
         // Remove tolerance to rescale values:
         if (axisX > 0) axisX -= VRC_INTTOLERANCE; else if (axisX < 0) axisX += VRC_INTTOLERANCE;         
         if (axisY > 0) axisY -= VRC_INTTOLERANCE; else if (axisY < 0) axisY += VRC_INTTOLERANCE;         
         if (axisZ > 0) axisZ -= VRC_INTTOLERANCE; else if (axisZ < 0) axisZ += VRC_INTTOLERANCE;         
         if (axisW > 0) axisW -= VRC_INTTOLERANCE; else if (axisW < 0) axisW += VRC_INTTOLERANCE;         
   
         // Rescale between -1.0 & 1.0:
         float _axisX = ((float) (axisX-VRC_AXISX_MIN))/ (float) abs(VRC_AXISX_MAX-VRC_AXISX_MIN);          
         _axisX = pow(2.0f, VRC_LINEARITY)*(pow(abs(_axisX-0.5f), VRC_LINEARITY));               
         if (axisX < 0) _axisX = -_axisX;   
   
         float _axisY = ((float) (axisY-VRC_AXISY_MIN))/ (float) abs(VRC_AXISY_MAX-VRC_AXISY_MIN);          
         _axisY = pow(2.0f, VRC_LINEARITY)*(pow(abs(_axisY-0.5f), VRC_LINEARITY));               
         if (axisY < 0) _axisY = -_axisY;   
   
         float _axisZ = ((float) (axisZ-VRC_AXISZ_MIN))/ (float) abs(VRC_AXISZ_MAX-VRC_AXISZ_MIN);          
         _axisZ = pow(2.0f, VRC_LINEARITY)*(pow(abs(_axisZ-0.5f), VRC_LINEARITY));               
         if (axisZ < 0) _axisZ = -_axisZ;   
   
         float _axisW = ((float) (axisW-VRC_AXISW_MIN))/ (float) abs(VRC_AXISW_MAX-VRC_AXISW_MIN);          
         _axisW = pow(2.0f, VRC_LINEARITY)*(pow(abs(_axisW-0.5f), VRC_LINEARITY));               
         if (axisW < 0) _axisW = -_axisW;      
   
         // Additional controls:
         float strafe = 0.0f;   
         if (SDL_JoystickGetButton(joystick, 7)) strafe += VRtranslationSpeed * VRC_TRANSLATIONSPEED * fpsFactor;
         if (SDL_JoystickGetButton(joystick, 6)) strafe -= VRtranslationSpeed * VRC_TRANSLATIONSPEED * fpsFactor;
         float height = 0.0f;
         if (SDL_JoystickGetButton(joystick, 5)) height += VRtranslationSpeed * VRC_TRANSLATIONSPEED * fpsFactor;
         if (SDL_JoystickGetButton(joystick, 4)) height -= VRtranslationSpeed * VRC_TRANSLATIONSPEED * fpsFactor;
   
         if (SDL_JoystickGetButton(joystick, 0)) VRtranslationSpeed = 1.0f;
         if (SDL_JoystickGetButton(joystick, 1)) VRtranslationSpeed = 2.0f;
         if (SDL_JoystickGetButton(joystick, 2)) VRtranslationSpeed = 4.0f;
         if (SDL_JoystickGetButton(joystick, 3)) VRtranslationSpeed = 8.0f;   
   
         if (SDL_JoystickGetButton(joystick, 10)) _axisY = -1.0f;
         if (SDL_JoystickGetButton(joystick, 11)) _axisY = -1.0f;            
   
         float _rspeed = VRrotationSpeed * VRC_ROTATIONSPEED * fpsFactor;
         float _tspeed = VRtranslationSpeed * VRC_TRANSLATIONSPEED * fpsFactor;
         VRcamera->rotate(_axisW*_rspeed, -_axisZ*_rspeed, 0.0f);   
         VRcamera->move(0.0f, height, 0.0f);
         VRcamera->moveDoom(_axisX*_tspeed+strafe, 0.0f, _axisY*_tspeed);
   }          
   
   // Contraints:
   MVVECTOR3 camRotation = VRcamera->getRotation();
   if (camRotation.x > VRC_CONSTRAINTANGLEX) camRotation.x = VRC_CONSTRAINTANGLEX;
   if (camRotation.x < -VRC_CONSTRAINTANGLEX) camRotation.x = -VRC_CONSTRAINTANGLEX;
   VRcamera->setRotation(camRotation);
         
   // Done:
   return true;         
}
#endif